/*
 * CFileMovieCore.cpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * This file was taken from Audioswitch package and modified in order
 * to work with samygodca
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "CFileMovieCore.hpp"

#define FUNC_FILEMOVIECORE_IN "[CFileMovieCore] %s In"
#define FUNC_FILEMOVIECORE_OUT "[CFileMovieCore] %s Out"

/*
int CFileMovieCore::EndPlay(void *)
int CFileMovieCore::BeginPlay(void *)
void CFileMovieCore::ErrorPlay(CMovieTDIF::_ErrorInfo, void *)
void CFileMovieCore::UnmuteCallback(TDiMpeg::TDAVDec_VideoInfo_t, void *)
void CFileMovieCore::ResChangeCallback(TDiMpeg::TDAVDec_VideoInfo_t, void *)
void CFileMovieCore::CallbackChaptering(unsigned int, unsigned int, MediaCommon::CaptureEncoding, void *, void (*)(unsigned int, unsigned int, unsigned int, MediaCommon::CaptureEncoding, unsigned int, unsigned char *, void *))
int CFileMovieCore::Close(void);
int CFileMovieCore::GetCurPosition(void)
*/
int CFileMovieCore::Skip(int offset, long long param2, long long param3, long long * param4)
{
	int cur_position_offset;
	int currenttime;

	PRINTF(FUNC_FILEMOVIECORE_IN, "Skip"); 
	if( m_bPlayMode == 0 ){
		ROSE_PRINT(0x4000,"[%s:%d]m_bPlayMode Error","Skip",__LINE__);
		return 0;
	}
	if( m_CMovieTDIF == NULL )
		return 0;

	if( offset < (int)0xC4653600 ){
		ROSE_PRINT(0x4, "[CFileMovieCore::Skip:%d]", __LINE__ );
		cur_position_offset = 0;
	} else {
		currenttime = m_CMovieTDIF->GetCurrentTime();
		ROSE_PRINT(0x4,"[CFileMovieCore::Skip:%d] currenttime = %d, totaltime = %d, offset = %d",
			__LINE__, currenttime, m_MovieSettings.Duration, offset );
		if( offset == 0xCCCCCD0 ){
			if( (m_MovieSettings.Duration - currenttime) < 6000 )
				return 0;
			cur_position_offset = m_MovieSettings.Duration - 5000;
			ROSE_PRINT(0x4,"cur_position_offset = %d", cur_position_offset );
		} else {
			cur_position_offset = m_MovieSettings.Duration - 3000;
			if( (currenttime <= cur_position_offset ) &&
				(offset > 0) )
				return 0;
			if( m_MovieSettings.Duration > (currenttime+(offset*1000)) ){
				if( m_MovieSettings.VideoFormat_TD == 2)
					cur_position_offset = (offset*1000) - 3000 + currenttime;
				else
					cur_position_offset = currenttime+(offset*1000);
			}
		}
		if( cur_position_offset < 0 )
			cur_position_offset = 0;
	}
	m_AudioVideoPlaying = false;
	StillVideo(true);
	SetSpeaker(false);
	if( m_CMovieTDIF->Stop(true,true) == 0 ){
		ROSE_PRINT(0x4,"###[CFileMovieCore][ERR RETURN], [%s, %s, %d]",
			__FILE__, "Skip", __LINE__ );
		m_iPlayerMode = 8;
		StillVideo(false);
		return 0;
	}
	if( m_CMovieTDIF->MovePosition(cur_position_offset, (long long unsigned int*)param4, false) == 0 ){
		ROSE_PRINT(0x4,"###[CFileMovieCore][ERR RETURN], [%s, %s, %d]",
			__FILE__, "Skip", __LINE__ );
		m_iPlayerMode = 8;
		StillVideo(false);
		return 0;
	}
	m_AudioVideoPlaying = true;
	if( m_CMovieTDIF->Play(true,true,true) == 0 ){
		ROSE_PRINT(0x4,"###[CFileMovieCore][ERR RETURN], [%s, %s, %d]",
			__FILE__, "Skip", __LINE__ );
		m_iPlayerMode = 8;
		StillVideo(false);
		return 0;
	}
	ROSE_PRINT(0x4, "SetCurPosition offset = %d", cur_position_offset );
	SendMessage( 0x49401, 6, 0);
	PRINTF(FUNC_FILEMOVIECORE_OUT, "Skip"); 
	return 1;
}

/*
int CFileMovieCore::Pause(void)
int CFileMovieCore::Play(Multimedia::VIDEO_INFO *, Multimedia::AUDIO_INFO *, unsigned char *, MediaCommon::DRM_INFO *, DecodeBuffer *)
int CFileMovieCore::Open(char *, Multimedia::_tMediaInfoContext *)
CFileMovieCore::~CFileMovieCore()
CFileMovieCore::CFileMovieCore(void)
void * CFileMovieCore::Create(void)
*/
int CFileMovieCore::Stop(void)
{
	int retval = 0;
	PRINTF(FUNC_FILEMOVIECORE_IN, "Stop");
	if( m_CMovieTDIF ){
		PCMem * pcmem = NULL;
		pcmem->Set( &VideoInfo, 0, sizeof(TDiMpeg::TDAVDec_VideoInfo_t) );
		pcmem->Set( &m_Display, 0, sizeof(_tDisplayInfo) );
		m_bRecalcDispSize = 0;
		if( m_MovieAlarm.mRes28 == 0x12C272B )
			m_MovieAlarm.Reset( 200, 0 );

		if( !m_bPlayMode ){
			ROSE_PRINT(0x4000, "[%s:%d]m_bPlayMode Error", "Stop", __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		MuteVideo( 1 );
		StillVideo( 0 );
		m_AudioVideoPlaying = 0;
		SetSpeaker( 0 );
		if( m_CMovieTDIF->InstallResChangeCallback( NULL, NULL ) == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - InstallResChangeCallback() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		if( m_CMovieTDIF->Stop( 1, 1 ) == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - Stop() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;		
		}
		SoundDisconnect();
		TDiVideoEnhancer * videoenhancer;
		videoenhancer = TDBuilder::GetTDiVideoEnhancer(0);
		//videoenhancer->TDiVideoEnhancerPureVirt5( mRes020 ); /* ??? what's going on here ???*/
		videoenhancer->SetPictureInformation( NULL );
		PRINTF("TDiVideoEnhancerPureVirt5 = 0x%08X", *((int *)videoenhancer+0x28));
		if( m_CMovieTDIF->InstallUnmuteCallback( NULL, NULL ) == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - InstallUnmuteCallback() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		if( m_CMovieTDIF->Disconnect() == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - Disconnect() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		m_iPlayerMode = 2;
		m_bPlayMode = 0;
		m_Display.Top = 0;
		m_Display.Left = 0;
		if( m_PanelResolution == 1 ){
			m_Display.Width	= PANNEL_768_V;
			m_Display.Height = PANNEL_1366_H;
		} else {
			m_Display.Width = PANNEL_V;
			m_Display.Height = PANNEL_H;
		}
		SendMessage( 0x49401, 2, 0 );
		retval = 1;
	}
CMovieCore_Stop_Out:
	PRINTF(FUNC_FILEMOVIECORE_OUT, "Stop");
	return retval;
}

/*
int CFileMovieCore::Forward(float)
int CFileMovieCore::Resume(void)

*/
