/*
 * CMovieCore.hpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */ 

#ifndef __CMOVIECORE_HPP__
#define __CMOVIECORE_HPP__

#include "Config.h"
#include "Multimedia.hpp"
#include "CMovieTDIF.hpp"
#include "PCHandler.hpp"
#include "TDiMpeg.hpp"
#include "PCMem.hpp"
#include "PCAlarm.hpp"

/* #include "CoreManager.hpp" */

#define FUNC_MOVIECORE_IN "[CFileMovieCore] %s In"

typedef unsigned char DecodeBuffer; 

class LIB_EXPORT CMovieCore
{
private:
public:
	PUBLIC static int m_PanelResolution;
	PUBLIC static int PANNEL_H;
	PUBLIC static int PANNEL_V;
	PUBLIC static int PANNEL_1366_H;
	PUBLIC static int PANNEL_768_V;

	struct _tDisplayInfo {
		int Top;								// 0x030
		int Left;								// 0x034
		int Height;								// 0x038
		int Width;								// 0x03C
	};

	PCHandler m_PCHandler;						// 0x004
	int mRes001[0x04];							// 0x00C
	int mRes01C;
	int mRes020;
	int mRes024;
	int mRes028;
	bool m_bPlayMode;							// 0x02C
	_tDisplayInfo m_Display;					// 0x030...0x040
	CMovieTDIF::_PositionInfo * PositionInfo;	// 0x040
	CMovieTDIF	* m_CMovieTDIF;					// 0x044
	TDiMpeg::TDAVDec_VideoInfo_t VideoInfo;		// 0x048
	int mRes002;								// 0x064
	CMovieTDIF::MovieSettings m_MovieSettings;	// 0x068
		/*
		int ContainerFormat_TD;						// 0x068
		int VideoFormat_TD;							// 0x06C
		int Audioformat_TD;							// 0x070
		_tSelAudioPCM m_BufPCM;						// 0x074
		_tSelAudioAAC m_BufAAC;						// 0x090
		_tSelAudioWMA m_BufWMA;						// 0x0A4
		int	mRes0B8;								// 0x0B8
		int mRes0BC;								// 0x0BC
		int64_t Filesize;							// 0x0C0
		int Duration;								// 0x0C8
		int	FrameWidth;								// 0x0CC
		int FrameHeight;							// 0x0D0
		int mRes0D4;
		int FrameRate;								// 0x0D8
		int mRes0DC;
		int SampleRate_TD;							// 0x0E0
		int mRes0E4;
		*/
	int mRes0E8;
	int mRes006;
	unsigned int m_iPlayerMode;				// 0x0F0
	bool m_bRecalcDispSize;					// 0x0F4
	int	m_iError;							// 0x0F8
	PCAlarm m_MovieAlarm;					// 0x0FC
	bool m_AudioVideoPlaying;				// 0x160

	static int m_CurrentDisplaySize;
	virtual ~CMovieCore( void );
	virtual int Open(char *, Multimedia::_tMediaInfoContext *);
	virtual int Play(Multimedia::VIDEO_INFO *, Multimedia::AUDIO_INFO *, unsigned char *, MediaCommon::DRM_INFO *, DecodeBuffer *);
	virtual int Stop(void);
	virtual int Pause(void);
	virtual int Resume(void);
	virtual int Close(void);
	virtual int GetCurPosition(void);
	virtual int GetDuration(void);
	virtual int CanTrick(void);
	virtual int CanSeek(void);
	virtual int SetEventReceiver(PCTask *);
	virtual int Skip(int, long long, long long, long long *);
	virtual int Forward(float);
	virtual int Rewind(float);
	virtual int SetAudioInfo(Multimedia::AUDIO_INFO *);
	virtual int GetPlayerRate(void);
	virtual int SetPlayerMode(int);
	virtual int GetPlayerMode(void);
	virtual int GetError(void);
	virtual int GetDisplaySize(MediaCommon::DisplayOutputArea * );
	virtual int SetDisplaySize(MediaCommon::DisplayOutputArea  const&);
	virtual int CallbackChaptering(unsigned int, unsigned int, MediaCommon::CaptureEncoding, void *, void (*)(unsigned int, unsigned int, unsigned int, MediaCommon::CaptureEncoding, unsigned int, unsigned char *, void *));
	virtual void SetEOS(bool);
	virtual int GetEOS(void);
	virtual int SendMessage(int, int, int);
	virtual int GetBufferSize(void);
	virtual void StopDecode(void);
	virtual void StartDecode(void);
	virtual int GetUnderflow(void);
	virtual int SetCaptionCallback(void (*)(MediaCommon::CAPTION_DATA *, void *), void *);
	virtual int SetCaptionStreamID(unsigned int);
	virtual int SetAudioStreamID(Multimedia::_tMediaInfoContext *, unsigned int);
	virtual int t_OnEvent(PTEvent const*);
	int EndPlay( void );
	int BeginPlay(void);
	int ErrorPlay(CMovieTDIF::_ErrorInfo, void *);
	int SendMediaTime(void);
	int CalculateDisplaySize(TDiMpeg::TDAVDec_VideoInfo_t);
	int SetVideoInfo(TDiMpeg::TDAVDec_VideoInfo_t);
	PUBLIC int StillVideo(bool);
	int ConvertSampleRateToTD(int);
	int AudioAACSampeRateIndex(int);
	int AudioADPCMFormatToTD(int);
	int AudioPCMFormatToTD(int);
	SdAudioFormat_k AudioFormatToTD(Multimedia::AudioCodecType);
	int SelectAudio(Multimedia::_tMediaInfoContext *, unsigned int);
	SdVideoFormat_k VideoFormatToTD(Multimedia::VideoCodecType, int);
	SdContainerFormat_k ContainerTypeToTD(Multimedia::ContainerType);
	int SetMediaInfo(Multimedia::_tMediaInfoContext *);
	int SetMediaInfo(Multimedia::_tMediaInfoContext *, int ); /* this method is required for audio switching in samygodca */
	int CleanParam(void);
	int GetConnectInfo(void);
	PUBLIC int MuteVideo(bool);
	int ResChangeCallback(TDiMpeg::TDAVDec_VideoInfo_t, void *);
	PUBLIC int SetSpeaker(bool);
	int UnmuteCallback(TDiMpeg::TDAVDec_VideoInfo_t, void *);
	PUBLIC int SoundDisconnect(void);
	int SoundConnect(void);
	void CreateTimerAlarm(void);
	CMovieCore( void );

};

#endif

