/*
 * CMovieCore.cpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "CMovieCore.hpp"
#include "Logging.h"

#ifdef __DCATOPCM__
extern enum CodecID soft_codec_id;
#endif

int CMovieCore::ConvertSampleRateToTD(int sample_rate)
{
	PRINTF(FUNC_MOVIECORE_IN, "ConvertSampleRateToTD");
	PRINTF("Input sample rate = %d", sample_rate);
	switch( sample_rate ){
	case 8000:
		return 0x01;
	case 11025:
	case 11024:
		return 0x02;
	case 12000:
		return 0x03;
	case 16000:
		return 0x04;
	case 22050:
		return 0x05;
	case 24000:
		return 0x06;
	case 32000:
		return 0x07;
	case 44100:
		return 0x08;
	case 48000:
		return 0x09;
	default:
		ROSE_PRINT(0x4000, "Not supported sampling rates : %d", sample_rate);
		return 0;
	}
}

int CMovieCore::AudioAACSampeRateIndex(int sample_rate)
{
	PRINTF(FUNC_MOVIECORE_IN, "AudioAACSampeRateIndex");
	PRINTF("Input sample rate = %d", sample_rate);
	switch( sample_rate ){
	case 11025:
		return 0x0A;
	case 12000:
		return 0x09;
	case 16000:
		return 0x08;
	case 22050:
		return 0x07;
	case 24000:
		return 0x06;
	case 32000:
		return 0x05;
	case 44100:
		return 0x04;
	case 48000:
		return 0x03;
	case 64000:
		return 0x02;
	case 88200:
		return 0x01;
	case 96000:
		return 0x00;
	default:
		ROSE_PRINT(0x4000, "Not Supported AAC Sample Rate!");
		return 0;
	}
}

int CMovieCore::AudioADPCMFormatToTD(int adpcm_format)
{
	PRINTF(FUNC_MOVIECORE_IN, __FUNCTION__);
	PRINTF("Input format = %d", adpcm_format);
	
	switch( adpcm_format ){
	case CODEC_ID_ADPCM_IMA_WAV:
		return 0x02;
	case CODEC_ID_ADPCM_MS:
		return 0x07;
	default:
		ROSE_PRINT(0x4000, "Not Supported ADPCM Format!");
		return 0;
	}
}

int CMovieCore::AudioPCMFormatToTD(int pcm_format)
{
	PRINTF(FUNC_MOVIECORE_IN, __FUNCTION__);
	PRINTF("Input format = %d", pcm_format);
	switch( pcm_format ){
	case CODEC_ID_PCM_S16LE:
		return 0x01;
	case CODEC_ID_PCM_S16BE:
		return 0x02;
	case CODEC_ID_PCM_U16LE:
		return 0x03;
	case CODEC_ID_PCM_U16BE:
		return 0x04;
	case CODEC_ID_PCM_S8:
		return 0x05;
	case CODEC_ID_PCM_U8:
		return 0x06;
	case CODEC_ID_PCM_MULAW:
		return 0x07;
	case CODEC_ID_PCM_ALAW:
		return 0x08;
	case CODEC_ID_PCM_S32LE:
		return 0x09;
	case CODEC_ID_PCM_S32BE:
		return 0x0A;
	case CODEC_ID_PCM_U32LE:
		return 0x0B;
	case CODEC_ID_PCM_U32BE:
		return 0x0C;
	case CODEC_ID_PCM_S24LE:
	case CODEC_ID_PCM_U24LE:
		return 0x0D;
	case CODEC_ID_PCM_S24BE:
		return 0x0E;
	case CODEC_ID_PCM_U24BE:
		return 0x10;
	case CODEC_ID_PCM_S24DAUD:
		return 0x11;
	default:
		ROSE_PRINT(0x4000, "Not Supported PCM Format!");
		return 0;
	}

}

SdAudioFormat_k CMovieCore::AudioFormatToTD(Multimedia::AudioCodecType acodec)
{
	PRINTF(FUNC_MOVIECORE_IN, "AudioFormatToTD");
	PRINTF("Input AudioCodecType = %d", acodec);
	switch( acodec ){
	case Multimedia::AUDIO_01:
	case Multimedia::AUDIO_OTHERS:
		return SDAUDIOFORMAT_OTHERS; // 0x0F;
	case Multimedia::AUDIO_MP3:
	case Multimedia::AUDIO_04:
		return SDAUDIOFORMAT_MP3; // 0x03;
	case Multimedia::AUDIO_AAC:
		return SDAUDIOFORMAT_AAC; // 0x06;
	case Multimedia::AUDIO_AC3:
		return SDAUDIOFORMAT_AC3; // 0x07;
	case Multimedia::AUDIO_DTS:
		return SDAUDIOFORMAT_DTS; // 0x0A;
	case Multimedia::AUDIO_PCM_S16:
	case Multimedia::AUDIO_XLAW:
		return SDAUDIOFORMAT_PCM; // 0x01;
	case Multimedia::AUDIO_WMA:
		return SDAUDIOFORMAT_WMA; // 0x0C;
	case Multimedia::AUDIO_ADPCM:
		return SDAUDIOFORMAT_ADPCM; // 0x02;
	case Multimedia::AUDIO_EAC3:
		return SDAUDIOFORMAT_EAC3; // 0x09;
	case Multimedia::AUDIO_HEAAC:
		return SDAUDIOFORMAT_HEAAC; // 0x08;
	case Multimedia::AUDIO_VORBIS:
		return SDAUDIOFORMAT_VORBIS; // 0x11;
	default:
		ROSE_PRINT(0x4000, "Not Supported Audio codec type!");
		return SDAUDIOFORMAT_NONE;
	}
}

SdVideoFormat_k CMovieCore::VideoFormatToTD(Multimedia::VideoCodecType vcodec, int dummy)
{
	PRINTF(FUNC_MOVIECORE_IN, "VideoFormatToTD");
	PRINTF("Input VideoCodecType = %d", vcodec);
	switch( vcodec ){
	case Multimedia::VIDEO_MSMPEGV3:// 0x03,
	case Multimedia::VIDEO_DIVX:	// 0x0B,
	case Multimedia::VIDEO_DX50:	// 0x0C,
		return SDVIDEOFORMAT_DIVX; // 5;
	case Multimedia::VIDEO_MPEG1:	// 0x04,
		return SDVIDEOFORMAT_MPEG1; // 1;
	case Multimedia::VIDEO_MPEG2:	// 0x05,
		return SDVIDEOFORMAT_MPEG2; // 2;
	case Multimedia::VIDEO_MPEG4:	// 0x06,
		return SDVIDEOFORMAT_MPEG4; // 3;
	case Multimedia::VIDEO_H264:	// 0x07,
		return SDVIDEOFORMAT_H264; // 4;
	case Multimedia::VIDEO_VC1:		// 0x08,
	case Multimedia::VIDEO_WMV9:	// 0x0A,
		return SDVIDEOFORMAT_WMV9; // 6;
	case Multimedia::VIDEO_XVID:	// 0x0D,
		return SDVIDEOFORMAT_XVID; // 7;
	case Multimedia::VIDEO_MJPEG:	// 0x0F,
		return SDVIDEOFORMAT_MJPEG; // 8;
	case Multimedia::VIDEO_MJPEGB:  // 0x10,
	case Multimedia::VIDEO_WMV:		// 0x09,
	case Multimedia::VIDEO_14: 
	default:
		return SDVIDEOFORMAT_NONE; // 0;
	}
}

SdContainerFormat_k CMovieCore::ContainerTypeToTD(Multimedia::ContainerType ccodec)
{
	PRINTF(FUNC_MOVIECORE_IN, "ContainerTypeToTD");
	PRINTF("Input ContainerType = %d", ccodec);
	switch( ccodec ){
	case Multimedia::CONTAINER_1:		// 0x01,
		return SDCONTAINERFORMAT_01; // 1;
	case Multimedia::CONTAINER_3:		// 0x03,
		return SDCONTAINERFORMAT_02; // 2;
	case Multimedia::CONTAINER_MPEG:	// 0x04,
	case Multimedia::CONTAINER_8:		// 0x08,
		return SDCONTAINERFORMAT_MPEG; // 3;
	case Multimedia::CONTAINER_MPEGTS:	// 0x05,
		mRes0E8 = 0xBC;
		return SDCONTAINERFORMAT_MPEGTS; // 4;
	case Multimedia::CONTAINER_ASF:		// 0x06,
		return SDCONTAINERFORMAT_ASF; // 5;
	case Multimedia::CONTAINER_7:		// 0x07,
		return SDCONTAINERFORMAT_06; // 6;
	case Multimedia::CONTAINER_AVI:		// 0x09,
		return SDCONTAINERFORMAT_AVI; // 8;
	case Multimedia::CONTAINER_10:		// 0x0A,
		return SDCONTAINERFORMAT_09; // 9;
	case Multimedia::CONTAINER_MP3:		// 0x0B,
		return SDCONTAINERFORMAT_MP3; // 10;
	case Multimedia::CONTAINER_MP4:		// 0x0C,
		return SDCONTAINERFORMAT_MP4; // 11;
	case Multimedia::CONTAINER_MPEGTTS:	// 0x0D,
		mRes0E8 = 0xC0;
		return SDCONTAINERFORMAT_MPEGTS; // 4;
	case Multimedia::CONTAINER_MKV:		// 0x0E,
		return SDCONTAINERFORMAT_MKV;	// 15;
	case Multimedia::CONTAINER_15:		// 0x0F,
		return SDCONTAINERFORMAT_16; // 16;
	default:
		return SDCONTAINERFORMAT_NONE;
	}
}

int CMovieCore::SetMediaInfo(Multimedia::_tMediaInfoContext * media_info, int audio_stream_index)
{
	int target_video_stream;
	int target_audio_stream;
	int counter_video_streams;
	int counter_audio_streams;
	int frame_width;
	int frame_height;
	float frame_rate;
	SdVideoFormat_k videoformat_TD;
	SdAudioFormat_k audioformat_TD;
	int sample_rate;
	int audio_index;
	int video_index;
	CMovieTDIF::_tSetAudioPCM SettingsPCM;
	CMovieTDIF::_tSetAudioWMA SettingsWMA;
	CMovieTDIF::_tSetAudioAAC SettingsAAC;

	PRINTF(FUNC_MOVIECORE_IN, __FUNCTION__);

#if 0
	if( media_info->nVideoStream > 0 ){
		target_video_stream = 1;
		target_audio_stream = 1;
	} else {
		target_audio_stream = media_info->nAudioStream;
		target_video_stream = media_info->nVideoStream;
	}
#else
	if( media_info->nVideoStream > 0 ){
		target_video_stream = 1;
	} else {
		target_video_stream = media_info->nVideoStream;
	}
	if( audio_stream_index > media_info->nAudioStream )
		target_audio_stream = media_info->nAudioStream;
	else
		target_audio_stream = audio_stream_index;
#endif

	memset( (void *)&SettingsPCM, 0, sizeof(CMovieTDIF::_tSetAudioPCM) );
	memset( (void *)&SettingsWMA, 0, sizeof(CMovieTDIF::_tSetAudioWMA) );
	memset( (void *)&SettingsAAC, 0, sizeof(CMovieTDIF::_tSetAudioAAC) );

	if( media_info->nStreams > 0 ){
		counter_audio_streams = 0;
		frame_height = 0;
		frame_rate = 0;
		counter_video_streams = 0;
		frame_width = 0;
		videoformat_TD = SDVIDEOFORMAT_NONE;
		audioformat_TD = SDAUDIOFORMAT_NONE;
		sample_rate = 0;
		audio_index = 0;
		video_index = 0;

		for(int i = 0; i < media_info->nStreams; i++ ){
			switch( media_info->stream_info[i].stream_type ){
				case Multimedia::STREAM_TYPE_VIDEO:
					counter_video_streams++;
					if( (counter_video_streams == target_video_stream)
#ifndef __DCATOPCM__
						|| (media_info->stream_info[i].stream_id < video_index ) 
#endif
						){
						video_index = media_info->stream_info[i].stream_id;
						videoformat_TD = VideoFormatToTD(media_info->stream_info[i].video_info.VideoCodecID,
							media_info->stream_info[i].video_info.nSubId);
						frame_width = media_info->stream_info[i].video_info.nFrameWidth;
						frame_height = media_info->stream_info[i].video_info.nFrameHeight;
						frame_rate = media_info->stream_info[i].video_info.nFrameRate;
						ROSE_PRINT(0x4, "In handleMediaInfo_forSDAL, pMediaInfo->tStreams[nIndex].tVideoInfo.nPid = 0x%x",
							media_info->stream_info[i].video_info.nPid );
					}
					break;
				case Multimedia::STREAM_TYPE_AUDIO:
					counter_audio_streams++;
					if( (counter_audio_streams == target_audio_stream)
#ifndef __DCATOPCM__
						|| (media_info->stream_info[i].stream_id < audio_index )
#endif
						){
						audio_index = media_info->stream_info[i].stream_id;
						audioformat_TD = AudioFormatToTD(media_info->stream_info[i].audio_info.AudioCodecID);
						sample_rate = media_info->stream_info[i].audio_info.nSampleRate;
						switch( audioformat_TD ){
						case SDAUDIOFORMAT_PCM:	// 0x1:
						case SDAUDIOFORMAT_ADPCM:	// 0x2:
#ifdef __DCATOPCM__
							soft_codec_id = CODEC_ID_NONE;
#endif
							SettingsPCM.eFsRate = ConvertSampleRateToTD(sample_rate);
							if( media_info->stream_info[i].audio_info.nChannel && 
								sample_rate ){
								SettingsPCM.bitWidth = media_info->stream_info[i].audio_info.nBitRate / sample_rate / media_info->stream_info[i].audio_info.nChannel;
							}
							SettingsPCM.channels = media_info->stream_info[i].audio_info.nChannel;
							SettingsPCM.bLittleEndian = 1;
							if( audioformat_TD == SDAUDIOFORMAT_ADPCM ){
								SettingsPCM.blockAlign = media_info->stream_info[i].audio_info.nBlockAlign;
								SettingsPCM.eADPCM = AudioADPCMFormatToTD(media_info->stream_info[i].audio_info.nSubId);
								SettingsPCM.ePCM = 0;
							} else {
								SettingsPCM.blockAlign = 0;
								SettingsPCM.eADPCM = 0;
								SettingsPCM.ePCM = AudioPCMFormatToTD(media_info->stream_info[i].audio_info.nSubId);
							}
							break;
						case SDAUDIOFORMAT_AAC: // 0x6:
						case SDAUDIOFORMAT_HEAAC: // 0x8:
#ifdef __DCATOPCM__
							soft_codec_id = CODEC_ID_NONE;
#endif
							SettingsAAC.type = 0x4;
							SettingsAAC.samplerate_index = AudioAACSampeRateIndex(sample_rate);
							SettingsAAC.sub_id = media_info->stream_info[i].audio_info.nSubId;
							SettingsAAC.channels = media_info->stream_info[i].audio_info.nChannel;
							SettingsAAC.sample_rate = sample_rate;
							SettingsAAC.e8 = 0;
							SettingsAAC.e12 = 0;
							SettingsAAC.e13 = 0;
							SettingsAAC.e14 = 0;
							SettingsAAC.e15 = 0;
							if( SettingsAAC.sub_id == 0x5 ){
								SettingsAAC.e17 = media_info->stream_info[i].audio_info.mRes4;
								SettingsAAC.e18 = media_info->stream_info[i].audio_info.mRes5;
								SettingsAAC.e16 = 1;
							} else {
								SettingsAAC.e17 = 0;
								SettingsAAC.e18 = 0;
								SettingsAAC.e16 = 0;
							}
							break;
						case SDAUDIOFORMAT_WMA: // 0xC:
#ifdef __DCATOPCM__
							soft_codec_id = CODEC_ID_NONE;
#endif
							SettingsWMA.codec_tag = media_info->stream_info[i].audio_info.nCodecTag;
							SettingsWMA.bits_per_sample = media_info->stream_info[i].audio_info.nValidBitsPerSample;
							SettingsWMA.channels = media_info->stream_info[i].audio_info.nChannel;
							SettingsWMA.byte_rate = (media_info->stream_info[i].audio_info.nBitRate+7) >> 3;
							SettingsWMA.block_align = media_info->stream_info[i].audio_info.nBlockAlign << 3;
							SettingsWMA.sample_rate = sample_rate;
							break;
#ifdef __DCATOPCM__
						case SDAUDIOFORMAT_DTS:	// 0xA:
							PRINTF("%s: Setting PCM values for DCA audio.", __FUNCTION__ );
							soft_codec_id = CODEC_ID_DTS;
							sample_rate = 48000;
							SettingsPCM.eFsRate = ConvertSampleRateToTD(sample_rate);
							SettingsPCM.bitWidth = 16;
							SettingsPCM.channels = 2;
							SettingsPCM.bLittleEndian = 1;
							audioformat_TD = SDAUDIOFORMAT_PCM;
							SettingsPCM.blockAlign = 0;
							SettingsPCM.eADPCM = 0;
							SettingsPCM.ePCM = 1; // AudioPCMFormatToTD(CODEC_ID_PCM_S16BE);
							m_MovieSettings.mRes0B8 = 0;
							m_MovieSettings.mRes0BC = 0;
							break;
#endif
						default:
							break;
						}
					}
					break;
				default:
					break;
			}
		}
		m_MovieSettings.ContainerFormat_TD = ContainerTypeToTD(media_info->container_info.nId);
		if( m_MovieSettings.ContainerFormat_TD == SDCONTAINERFORMAT_NONE ){
			ROSE_PRINT(0x4004, "TD_CONTAINER_NONE");
			return 0;
		}
		m_MovieSettings.mRes0B8 = mRes0E8;
		m_MovieSettings.VideoFormat_TD = videoformat_TD;
		if( m_MovieSettings.VideoFormat_TD == 0 ){
			ROSE_PRINT(0x4004, "TD_VIDEO_FORMAT_NONE");
			return 0;
		}
		m_MovieSettings.AudioFormat_TD = audioformat_TD;
		m_MovieSettings.SampleRate_TD = ConvertSampleRateToTD(sample_rate);
		m_MovieSettings.Filesize = media_info->file_info.filesize;
		memcpy( (void *)&m_MovieSettings.eSettingsPCM, (void *)&SettingsPCM, sizeof(CMovieTDIF::_tSetAudioPCM) );
		memcpy( (void *)&m_MovieSettings.eSettingsAAC, (void *)&SettingsAAC, sizeof(CMovieTDIF::_tSetAudioAAC) );
		memcpy( (void *)&m_MovieSettings.eSettingsWMA, (void *)&SettingsWMA, sizeof(CMovieTDIF::_tSetAudioWMA) );
		m_MovieSettings.Duration = media_info->container_info.nTotalPlayTime * 1000;
		m_MovieSettings.FrameRate = frame_rate;
		m_MovieSettings.FrameWidth = frame_width;
		m_MovieSettings.FrameHeight = frame_height;
		return 1;
	} else {
		frame_height = 0;
		frame_rate = 0;
		frame_width = 0;
		videoformat_TD = SDVIDEOFORMAT_NONE;
		audioformat_TD = SDAUDIOFORMAT_NONE;
		sample_rate = 0;
		return 0;
	}
}
