/*
 * CMovieTDIF.cpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "CMovieTDIF.hpp"
#include "Logging.h"

#define SUPPORTED_CVA_COMBINATION	0
#define FAIL_UNSUPPORTED_CONTAINER1	1
#define FAIL_UNSUPPORTED_AUDIOCODEC	2
#define FAIL_UNSUPPORTED_CONTAINER3 3
#define FAIL_UNSUPPORTED_FRAMERATE	4
#define FAIL_UNSUPPORTED_CONTAINER5	5

int CheckSupportMovie(CMovieTDIF::MovieSettings settings)
{
	PRINTF( "container = %i, videoformat = %i, audioformat = %i", 
		settings.ContainerFormat_TD, settings.VideoFormat_TD, settings.AudioFormat_TD );

	switch( settings.ContainerFormat_TD ){
	case SDCONTAINERFORMAT_MPEG:				// 3
		switch( settings.VideoFormat_TD ){
		case SDVIDEOFORMAT_MPEG1:
		case SDVIDEOFORMAT_MPEG2:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_5:		// 5
			case SDAUDIOFORMAT_AC3:		// 7
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
				return SUPPORTED_CVA_COMBINATION;
			case SDAUDIOFORMAT_PCM:		// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_AAC:		// 6
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC; // 2;
			}
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	
			case SDAUDIOFORMAT_PCM:
			case SDAUDIOFORMAT_ADPCM:
			case SDAUDIOFORMAT_MP3:
			case SDAUDIOFORMAT_4:
			case SDAUDIOFORMAT_5:
			case SDAUDIOFORMAT_AC3: // 7
				return FAIL_UNSUPPORTED_CONTAINER1;
			case SDAUDIOFORMAT_AAC:	// 6
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;
			}
		}
		break;
	case SDCONTAINERFORMAT_MPEGTS:	// 4
		switch( settings.VideoFormat_TD ){
		case SDVIDEOFORMAT_MPEG2:	// 2
		case SDVIDEOFORMAT_H264:	// 4
		case SDVIDEOFORMAT_WMV9:	// 6
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_PCM:		// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_AC3:		// 7
			case SDAUDIOFORMAT_HEAAC:	// 8
			case SDAUDIOFORMAT_EAC3:	// 9
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
				return SUPPORTED_CVA_COMBINATION;	// 0
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
			}
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_AC3:		// 7
			case SDAUDIOFORMAT_HEAAC:	// 8
			case SDAUDIOFORMAT_EAC3:	// 9
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
				return FAIL_UNSUPPORTED_CONTAINER1;
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;
			}
		}
		break;
	case SDCONTAINERFORMAT_ASF:		// 5
		switch( settings.VideoFormat_TD ){
		case SDVIDEOFORMAT_MPEG4:		// 3
		case SDVIDEOFORMAT_H264:		// 4
		case SDVIDEOFORMAT_DIVX:		// 5
		case SDVIDEOFORMAT_WMV9:		// 6
		case SDVIDEOFORMAT_XVID:		// 7
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_PCM:		// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_AC3:		// 7
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
			case SDAUDIOFORMAT_WMA:		// 12
				return SUPPORTED_CVA_COMBINATION;	// 0
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
			}
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_PCM:		// 1
			case SDAUDIOFORMAT_ADPCM:	// 2
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_AC3:		// 7
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
			case SDAUDIOFORMAT_WMA:		// 12
				return FAIL_UNSUPPORTED_CONTAINER1;
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;
			}
		}
		break;
	case SDCONTAINERFORMAT_AVI:		// 8
		switch( settings.VideoFormat_TD ){
		case SDVIDEOFORMAT_MPEG4:		// 3
		case SDVIDEOFORMAT_H264:		// 4
		case SDVIDEOFORMAT_DIVX:		// 5
		case SDVIDEOFORMAT_XVID:		// 7
		case SDVIDEOFORMAT_MJPEG:		// 8
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_PCM:		// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AC3:		// 7
				return SUPPORTED_CVA_COMBINATION;	// 0
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
			}
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_PCM:		// 1
			case SDAUDIOFORMAT_ADPCM:	// 2
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AC3:		// 7
				return FAIL_UNSUPPORTED_CONTAINER1;
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;
			}
		}
		break;
	case SDCONTAINERFORMAT_MP4:		// 11
		switch( settings.VideoFormat_TD ){
		case SDVIDEOFORMAT_MPEG4:
		case SDVIDEOFORMAT_H264:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_PCM:	// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_HEAAC:	// 7
				return SUPPORTED_CVA_COMBINATION;	// 0
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
			}
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_ADPCM:	// 2
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_HEAAC:	// 7
				return FAIL_UNSUPPORTED_CONTAINER1;	// 1
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;	// 3
			}
		}
		break;
	case SDCONTAINERFORMAT_MKV:		// 15
		switch( settings.VideoFormat_TD ){	
		case SDVIDEOFORMAT_MPEG4:
		case SDVIDEOFORMAT_H264:
		case SDVIDEOFORMAT_DIVX:	// 5
		case SDVIDEOFORMAT_XVID:	// 7
		case SDVIDEOFORMAT_MJPEG:	// 8
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_MP3:		// 3
			case SDAUDIOFORMAT_4:		// 4
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_AC3:		// 7
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
				return SUPPORTED_CVA_COMBINATION;
			case SDAUDIOFORMAT_PCM:		// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC; // 2;
			}
			break;
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_NONE:	
			case SDAUDIOFORMAT_PCM:
			case SDAUDIOFORMAT_ADPCM:
			case SDAUDIOFORMAT_MP3:
			case SDAUDIOFORMAT_4:
			case SDAUDIOFORMAT_AAC:	// 6
			case SDAUDIOFORMAT_AC3: // 7
#ifdef __DCATOPCM__
			case SDAUDIOFORMAT_DTS:		// 10
#endif
				return FAIL_UNSUPPORTED_CONTAINER1;
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;
			}
		}
		break;
	case SDCONTAINERFORMAT_16:		// 16
		switch( settings.VideoFormat_TD ){
		case SDVIDEOFORMAT_MPEG4:	// 3
		case SDVIDEOFORMAT_H264:	// 4
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_PCM:	// 1
				switch( settings.eSettingsPCM.ePCM ){
				case 0:
				case 1:
				case 6:
				case 7:
				case 8:
				case 13:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_ADPCM:	// 2
				switch( settings.eSettingsPCM.eADPCM ){
				case 7:
				case 2:
					return SUPPORTED_CVA_COMBINATION;	// 0
				default:
					return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
				}
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_HEAAC:	// 8
				return SUPPORTED_CVA_COMBINATION;	// 0
			default:
				return FAIL_UNSUPPORTED_AUDIOCODEC;	// 2
			}
		default:
			switch( settings.AudioFormat_TD ){
			case SDAUDIOFORMAT_ADPCM:	// 2
			case SDAUDIOFORMAT_NONE:	// 0
			case SDAUDIOFORMAT_AAC:		// 6
			case SDAUDIOFORMAT_HEAAC:	// 8
				return FAIL_UNSUPPORTED_CONTAINER1;	// 1
			default:
				return FAIL_UNSUPPORTED_CONTAINER3;	// 3
			}
		}
		break;
	default:
		return FAIL_UNSUPPORTED_CONTAINER3;
	}
}
