/*
 * SdAVDec.hpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __SDAVDEC_HPP__
#define __SDAVDEC_HPP__

#include "Config.h"
#include "Typedef.h"
#include "spITsdComm.h"
#include "spIMpeg.h"

enum SdAVDecID_k {
	DEC_AE0 = 0
};

enum SdAVDec_StatusMask_k {
	SdAVDec_StatusMask_DUMMY = 0
};

enum SdAVDec_DemuxOut_k {
	SdAVDec_DemuxOut_DUMMY = 0
};

enum SdExtInID_k {
	SdExtInID_DUMMY = 0
};

enum SdSource_k {
	SdSource_DUMMY = 0
};

enum SdAVDec_DemuxType_k {
	SdAVDec_DemuxType_DUMMY = 0
};

enum SdPowerState_k {
	SdPowerState_DUMMY = 0
};

enum SdVideoID_k {
	SdVideoID_k_DUMMY = 0
};

enum SlotState_k {
	SlotState_k_DUMMY = 0
};

enum SdAudioFsRate_k {
	SdAudioFsRate_DUMMY = 0
};

struct SdAVDec_Settings_t {
	int stream_type;			// 0x00
	int video_format;			// 0x04
	int audio_format;			// 0x08
	int video_pid;				// 0x0C
	int audio_pid;				// 0x10
	int mRes14;
	int pcr_pid;				// 0x18
	int audioDescription_pid;	// 0x1C
	int av_sync_on;				// 0x20
	int audioDescription_on;	// 0x24
	int mRes28[2];
	int mRes30[4];
	int mRes40[4];
	int mRes50[4];
	int mRes60[4];
	int mRes70[2];
	int eAudStereoMode;			// 0x78
	int mRes7C;
	int mRes80;
};

struct SdAVDec_Status_t {
	int mRes00;
};

struct SdAVDec_ErrorInfo_t {
	int mRes00;
};

struct SdAVDec_CaptureInfo_t {
	int mRes00;
};

struct SdAVDec_VideoInfo_t {
	int	mRes1;
	int	mRes2;
	int	mRes3;
	int	mRes4;
	int	mRes5;
	int	mRes6;
	int	mRes7;
};

struct SdAudOutputType_t {
	int mRes00;
};

typedef void * AVDecDpHndl_e;
typedef int Audio_SamplingFreq_e;
typedef int Audio_Codec_e;
typedef int Audio_SamplingFreq_e;


int SdAVDec_Debug(void);
int SdAVDec_AudioResume(SdAVDecID_k);
int SdAVDec_AudioPause(SdAVDecID_k);
int SdAVDec_VideoResume(SdAVDecID_k);
int SdAVDec_VideoPause(SdAVDecID_k);
int SdAVDec_ClipPlayStop(SdAVDecID_k);
int SdAVDec_ClipPlayStart(SdAVDecID_k, unsigned char *, unsigned int);
int SdAVDec_InstallCallback(SdAVDecID_k, void (*)(SdAVDec_Status_t *, SdAVDec_StatusMask_k, unsigned int), unsigned int, unsigned long, int);
int SdAVDec_Get(SdAVDecID_k, SdAVDec_Settings_t *);
int SdAVDec_TSOutStop(void *);
int SdAVDec_TSOutStart(void *);
int SdAVDec_InstallCallbackError(SdAVDecID_k, void (*)(SdAVDec_ErrorInfo_t *, unsigned int), unsigned int, int);
int SdAVDec_InstallCallbackCapture(SdAVDecID_k, void (*)(SdAVDec_CaptureInfo_t *, unsigned int), unsigned int, unsigned int);
int SdAVDec_StreamOutSetPid(void *, int *, unsigned int, SdAVDec_DemuxOut_k, int);
int SdAVDec_AudioStop(SdAVDecID_k);
int SdAVDec_VideoStop(SdAVDecID_k);
int SdAVDec_DemuxStop(void *, SdAVDec_DemuxOut_k);
int SdAVDec_DemuxStart(void *, SdAVDec_DemuxOut_k);
int SdAVDec_DemuxDeallocate(void *, SdAVDec_DemuxOut_k);
int SdAVDec_DecoderConnect(SdAVDecID_k, SdExtInID_k);
int SdAVDec_DecoderConnect(SdAVDecID_k, void *);
int SdAVDec_DemuxAllocate(void **, SdSource_k, SdAVDec_DemuxType_k, SdAVDec_DemuxOut_k, int);
int SdAVDec_VideoStart(SdAVDecID_k, int);
int SdAVDec_DecoderDisconnect(SdAVDecID_k);
int SdAVDec_GetStatus(SdAVDecID_k, unsigned long, SdAVDec_Status_t *, unsigned int);
int SdAVDec_AudioStart(SdAVDecID_k, int);
int SdAVDec_Set(SdAVDecID_k, SdAVDec_Settings_t *);
int SdAVDec_Init(SdPowerState_k);

struct CAVDec_InitParam_t {
	int mRes000;
};

class LIB_EXPORT CAVDec
{

	public:
		int mRes004;
		int mRes008;
		spIMpeg_Context_t * m_hMpeg;
		int mRes010[4];

		virtual ~CAVDec();
		virtual int demuxConnect(DemuxInfoHandle_t *,SdSource_k,int,int);;
		virtual int videoConnect(SdVideoID_k);
		virtual int demuxDisconnect(void);
		virtual int videoDisconnect(void);
		virtual int demuxStart(void);
		virtual int demuxStop(void);
		virtual int videoStart(int,int);
		virtual int videoStop(void);
		virtual int audioStart(int);
		virtual int audioStop(void);
		virtual int get(SdAVDec_Settings_t *);
		virtual int set(SdAVDec_Settings_t *);
		virtual int getStatus(ulong,SdAVDec_Status_t *);
		virtual int installCallback(void (*)(SdAVDec_Status_t *,SdAVDec_StatusMask_k,uint),uint,ulong,int);
		virtual int installCallbackCapture(void (*)(SdAVDec_CaptureInfo_t *,uint),uint,uint);
		virtual int installCallbackError(void (*)(SdAVDec_ErrorInfo_t *,uint),uint,int);
		virtual int clipPlayStart(uchar *,uint);
		virtual int clipPlayStop(void);
		virtual int videoPause(void);
		virtual int videoResume(void);
		virtual int audioPause(void);
		virtual int audioResume(void);
		virtual int locMpegStart(void);
		virtual int locMpegStop(void);
		virtual int locStopAVOut(uint,SlotState_k);
		virtual int setAudioLockStatus(uint,void *);
		virtual int locGetVideoLock(int);
		virtual int locGetVideoInfo(SdAVDec_VideoInfo_t *);
		virtual int locGetUnMuteCondition(int);
		virtual int locSetDpHandleId(AVDecDpHndl_e);
		virtual int connectHdmiAudio(int,SdExtInID_k);
		virtual int disconnectHdmiAudio(void);
		virtual int playHdmiAudio(void);
		virtual int playHdmiEs(SdAudioFsRate_k);
		virtual int playPcm(SdAudioFsRate_k);
		virtual int playWma(void);
		virtual int startI2sTx(uint,Audio_SamplingFreq_e,uchar,uchar);
		virtual int startSpdifTx(Audio_Codec_e,Audio_SamplingFreq_e,uint,uchar);
		virtual int stopSpdifTx(void);
		virtual int setAudioAvSync(int);
		virtual int getAudRxClockSource(uint);
		virtual int getAudTxClockSource(SdAudOutputType_t,uint);
		virtual int getAudTxSamplingRate(Audio_SamplingFreq_e);
		virtual int getAudRxEngineMap(uint);
		virtual int getAudTxEngineMap(SdAudOutputType_t,uint,int);

		CAVDec(SdAVDecID_k, CAVDec_InitParam_t *);
};

#endif

