/*
 * audioswitch_inj.cpp
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * This file was taken from Audioswitch package and modified in order
 * to work with samygodca
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "audioswitch_inj.hpp"
#include "Logging.h"

/* this contains the currently selected audio index (1...n) */
static int dca_currentstream = 0;
/* this contains the current filename to detect change */
static char dca_lastfilename[256];

static int * CMovie_Instance = 0;
static int * avfd;

extern enum CodecID soft_codec_id;

static int Multimedia_MediaPlayer_Movie_CMovie_SetStreamId(int * CMovie_this)
{
	if( CMovie_this == NULL )
		return 0;

	switch( CMovie_this[0x220/4] ){	/* container ID */
	case 0:
		return 0;
	case 4:
	case 8:
		switch( CMovie_this[0xD98/4] ){		/* video codec ID */
		case 5:
		case 4:
			switch( CMovie_this[0xDA8/4] ){	/* audio codec ID */
			case 3:
				CMovie_this[0xDB0/4] = 0xC0;
				break;
			case 9:
				CMovie_this[0xDB0/4] = 0xA0;
				break;
			}
			CMovie_this[0xDAC/4] = 0xE0;
			CMovie_this[0xDA0/4] = 0xE0;
			CMovie_this[0xD9C/4] = 0xE0;
			break;
		}
		break;
	case 11:
		CMovie_this[0xDAC/4] = 0;
		CMovie_this[0xDB0/4] = 0xC0;
		CMovie_this[0xDA0/4] = 0;
		CMovie_this[0xD9C/4] = 0;
		break;		
	default:
		break;
	}
	return 1;
}  

/* 
 This function in exeDSP will always be called with stream_index = 1
 (xref: Multimedia::MediaPlayer::Movie::CFileMovie::Play(long long,uchar *,MediaCommon::DRM_INFO *)
		Multimedia::MediaPlayer::Movie::CPumpMovie::Play(long long,uchar *,MediaCommon::DRM_INFO *)
*/
int Multimedia_MediaPlayer_Movie_CMovie_GetAudioInfo(int * CMovie_this, struct Multimedia::_tMediaInfoContext * media_info, int stream_index)
{
	int stream_to_select;
	int count_audio_streams;
//	int last_stream_id;
	int i;

	/* Store current CMovie instance for later access to its members */
	CMovie_Instance = CMovie_this;
	if( CMovie_Instance == NULL )
		return 0;

	/* instead of parameter stream_index we now use
	   the global variable */
	if( dca_currentstream > media_info->nAudioStream )
		stream_to_select = media_info->nAudioStream;
	else
		stream_to_select = dca_currentstream;

	memset( (void *)&CMovie_Instance[0x0DA8/4], 0, 16 );

	/*
	LOG("number of streams = [%d]", media_info[0x220/4]);
	LOG("number of audio streams = [%d]", media_info[0x228/4]);
	*/

	count_audio_streams = 0;
	for( i = 0; i < media_info->nStreams; i++ ){
		if( media_info->stream_info[i].stream_type != 3 )
			continue;
		count_audio_streams++;
		if( count_audio_streams == stream_to_select ){
			switch( media_info->stream_info[i].audio_info.AudioCodecID ){
			case Multimedia::AUDIO_DTS:
				soft_codec_id = CODEC_ID_DTS;
				break;
			default:
				soft_codec_id = CODEC_ID_NONE;
				break;
			}
			CMovie_Instance[0x0DA8/4] = media_info->stream_info[i].audio_info.AudioCodecID;
			CMovie_Instance[0x0DAC/4] = media_info->nPcrPid;
			CMovie_Instance[0x0DB0/4] = media_info->stream_info[i].audio_info.nPid;
			CMovie_Instance[0x0DB4/4] = i+1;					// StreamIndex
		}
	}
	LOG("audio codec = [%d] pcrpid = [%d] npid = [%d] index = [%d]", CMovie_Instance[0x0DA8/4],
		CMovie_Instance[0x0DAC/4], CMovie_Instance[0x0DB0/4], CMovie_Instance[0x0DB4/4]);
	return 1;

}

static AvfdAudFormat_k CFeeder_CH_locConvertSdalToAvfdAudFormat(SdAudioFormat_k eInfo)
{
	switch( eInfo ){
	case SDAUDIOFORMAT_PCM:
		return AVFDAUDFORMAT_PCM;	// 2;
	case SDAUDIOFORMAT_ADPCM:
		return AVFDAUDFORMAT_ADPCM;	// 1;
	case SDAUDIOFORMAT_MP3:
	case SDAUDIOFORMAT_5:
		return AVFDAUDFORMAT_MP3;	// 0;
	case SDAUDIOFORMAT_4:
		return AVFDAUDFORMAT_03;	// 3;
	case SDAUDIOFORMAT_AAC:
	case SDAUDIOFORMAT_HEAAC:
		return AVFDAUDFORMAT_AAC;	// 6;
	case SDAUDIOFORMAT_AC3:
		return AVFDAUDFORMAT_AC3;	// 4;
	case SDAUDIOFORMAT_WMA:			
		return AVFDAUDFORMAT_WMA;	// 5;
	default:
		PRINTF("[SdFEEDER]SdAudioFormat_k(): eInfo(%d) is unkown.", eInfo);
		return AVFDAUDFORMAT_SKIP;  // 8;
	}
} 

int CFileMovieCore2::Stop2( void )
{
	int retval = 0;
	PRINTF(FUNC_FILEMOVIECORE2_IN, "Stop2");
	if( m_CMovieTDIF ){
		PCMem * pcmem = NULL;
		pcmem->Set( &VideoInfo, 0, sizeof(TDiMpeg::TDAVDec_VideoInfo_t) );
		pcmem->Set( &m_Display, 0, sizeof(_tDisplayInfo) );
		m_bRecalcDispSize = 0;
		if( m_MovieAlarm.mRes28 == 0x12C272B )
			m_MovieAlarm.Reset( 200, 0 );

		if( !m_bPlayMode ){
			ROSE_PRINT(0x4000, "[%s:%d]m_bPlayMode Error", "Stop", __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		MuteVideo( 1 );
		StillVideo( 0 );
		m_AudioVideoPlaying = 0;
		SetSpeaker( 0 );
		if( m_CMovieTDIF->InstallResChangeCallback( NULL, NULL ) == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - InstallResChangeCallback() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		if( m_CMovieTDIF->Stop( 1, 1 ) == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - Stop() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;		
		}
		SoundDisconnect();
		TDiVideoEnhancer * videoenhancer;
		videoenhancer = TDBuilder::GetTDiVideoEnhancer(0);
		//videoenhancer->TDiVideoEnhancerPureVirt5( mRes020 ); /* ??? what's going on here ???*/
		videoenhancer->SetPictureInformation( NULL );
		PRINTF("TDiVideoEnhancerPureVirt5 = 0x%08X", *((int *)videoenhancer+0x28));
		if( m_CMovieTDIF->InstallUnmuteCallback( NULL, NULL ) == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - InstallUnmuteCallback() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		if( m_CMovieTDIF->Disconnect() == 0 ){
			m_iPlayerMode = 8;
			ROSE_PRINT(0x4000, "###[CFileMovieCore][ERR RETURN] [%s:%d] - Disconnect() failed.", __FUNCTION__, __LINE__ ); 
			goto CMovieCore_Stop_Out;
		}
		m_iPlayerMode = 2;
		m_bPlayMode = 0;
		m_Display.Top = 0;
		m_Display.Left = 0;
		if( m_PanelResolution == 1 ){
			m_Display.Width	= PANNEL_768_V;
			m_Display.Height = PANNEL_1366_H;
		} else {
			m_Display.Width = PANNEL_V;
			m_Display.Height = PANNEL_H;
		}
		SendMessage( 0x49401, 2, 0 );
		retval = 1;
	}
CMovieCore_Stop_Out:
	PRINTF(FUNC_FILEMOVIECORE2_OUT, "Stop2");
	return retval;
}

int CFileMovieCore2_Stop2( CFileMovieCore2 * Instance )
{
	CMovie_Instance = NULL;
	return Instance->Stop2();
}

/* replacement for CMovieCore::SetMediaInfo(...) */
int CMovieCore_SetMediaInfo( CMovieCore * Instance, Multimedia::_tMediaInfoContext * media_info )
{
	int retval;
	PRINTF("[%s] In", __FUNCTION__ );

	/* check whether filename had been changed */
	if( strcmp(dca_lastfilename, media_info->file_info.filename) ){
		PRINTF("[%s] New movie, resetting audio stream index.", __FUNCTION__); 
		dca_currentstream = 1;
		strncpy( dca_lastfilename, media_info->file_info.filename, sizeof(dca_lastfilename)-1 );
	} else {
		PRINTF("[%s] Same movie, setting audio stream index = %i", __FUNCTION__, dca_currentstream );
	}

	retval = Instance->SetMediaInfo(media_info, dca_currentstream);
	PRINTF("[%s] Out", __FUNCTION__ );
	return retval;
}

int
dca_process_keyproc(void)
{
	if (!CMovie_Instance)
		return (MOD_KEYPRESSED_UNKNOWN);

	// struct SdAVDec_Settings_t sdavdec_settings;
	struct Multimedia::_tMediaInfoContext * media_info = (struct Multimedia::_tMediaInfoContext *)&CMovie_Instance[0x18/4];
	CMovieCore * moviecore = NULL;
	SdAudioFormat_k sdaf_codec;
	int avfd_codec;

	dca_currentstream++;

	PRINTF("dca_currentstream [%i], media_info->nAudioStream [%i]", dca_currentstream, media_info->nAudioStream );
	if ( dca_currentstream > media_info->nAudioStream )
		{
			PRINTF( "returning to the first stream..\n" );
			dca_currentstream = 1;
		}

	Multimedia_MediaPlayer_Movie_CMovie_GetAudioInfo( CMovie_Instance,	media_info, dca_currentstream );
	Multimedia_MediaPlayer_Movie_CMovie_SetStreamId( CMovie_Instance );
	sdaf_codec = moviecore->AudioFormatToTD((Multimedia::AudioCodecType)CMovie_Instance[0x0DA8/4]); 
	avfd_codec = CFeeder_CH_locConvertSdalToAvfdAudFormat( sdaf_codec );

	LOG("Changing stream to [%d], nPid [%d], codec id [%d] ", dca_currentstream, CMovie_Instance[0x0DB0/4], avfd_codec ); 
	spIAvfd_ChangeAudio(0, CMovie_Instance[0x0DB0/4], avfd_codec );

	return (MOD_KEYPRESSED_PROCESSED);
}
