#include <dlfcn.h>
#include "dca.hpp"
#include "Config.h"
#include "samygodca_inj.hpp"
#include "audioswitch_inj.hpp"

struct dca_config config = {
	MKEY | KEY_MUTE /* audio_switch_key */,
	"", /* ffmpeg_location */
};

static struct mod_config_var config_vars[] = {
	{"AUDIO_SWITCH",    0, KEY_ENTRY(config.audio_switch_key)},
	{"FFMPEG_LOCATION", 0, PATH_ENTRY(config.ffmpeg_location)},
	{LAST_VAR_ENTRY}
};

int
mod_prepare(void *handle)
{
	snprintf(config.ffmpeg_location, sizeof(config.ffmpeg_location),
			 "%s/ffmpeg", get_app_path());
	return (EX_OK);
}

int
mod_init(void *handle)
{
	void *hExeDsp;
	unsigned int *lib_export;
	unsigned int old_opcodes[32];
	unsigned int new_opcodes[32];
#define ENTRY(fn, exedsp_name) (void *)(fn), exedsp_name
	struct {
		void *fn;
		const char *exedsp_name;
	} hooks[] = {
		/* DTS support injections */
		{ENTRY(CheckSupportMovie_DTS, "_Z17CheckSupportMovieN10CMovieTDIF13MovieSettingsE")},
		{ENTRY(CParsingMedia_PrepareParsing, "_ZN13CParsingMedia14PrepareParsingEv")},
		{ENTRY(CParsingMedia_GetMediaTypeInfo, "_ZN13CParsingMedia16GetMediaTypeInfoEPKcPN10Multimedia18_tMediaInfoContextE")},
		{ENTRY(CMovieCore_SetMediaInfo, "_ZN10CMovieCore12SetMediaInfoEPN10Multimedia18_tMediaInfoContextE")},
		{ENTRY(uldAvfd_InterfacerGetAudioCodecType_DTS, "uldAvfd_InterfacerGetAudioCodecType")},
		{ENTRY(staMkv_GetAudioCodecType_DTS, "staMkv_GetAudioCodecType")},

		/* audio switching injections */
		{ENTRY(Multimedia_MediaPlayer_Movie_CMovie_GetAudioInfo, "_ZN10Multimedia11MediaPlayer5Movie6CMovie12GetAudioInfoEPNS_18_tMediaInfoContextEi")},
		{ENTRY(CFileMovieCore2_Stop2, "_ZN14CFileMovieCore4StopEv")},
		{NULL, NULL}
	}, *hook;

	hExeDsp = dlopen( NULL, RTLD_LAZY);
	if (hExeDsp == NULL) {
		LOG("Cannot open exeDSP");
		return (EX_SOFTWARE);
	}

	if (samygodca_init(config.ffmpeg_location) != EX_OK)
		return (EX_SOFTWARE);

	LOG("Patching CheckSupportMovie()...");

	new_opcodes[0] = 0xE51FF004;
	for (hook = hooks; hook->fn != NULL; hook++) {
		new_opcodes[1] = (unsigned int)hook->fn;
		if (patch_opcodes(hExeDsp,
						  hook->exedsp_name, 0x00,
						  NULL, new_opcodes, NULL, 2,
						  PROT_READ | PROT_EXEC) != EX_OK) {
			LOG("Failed to patch[%s]", hook->exedsp_name);
		}
	}

	/* modify avfd interfacer functions for audio_codec with AVFDAUDFORMAT_PCM (2) */
	lib_export = (typeof(lib_export))
		dlsym(hExeDsp, "gAvfdInterfaceAudioCodecFunctions");
	if (lib_export != NULL) {
		memcpy(old_opcodes, lib_export + 2 * 25, sizeof(*lib_export) * 25);
		memcpy(new_opcodes, lib_export + 2 * 25, sizeof(*lib_export) * 25);

		new_opcodes[0] = (int)spIAe_Open_DTS;
		new_opcodes[1] = (int)spIAe_Close_DTS;
		new_opcodes[2] = (int)InterfacerAudioTransferData_DTS;
		new_opcodes[3] = (int)InterfacerAudioTransferDataWithSyncHdr_DTS;
		new_opcodes[4] = (int)InterfacerAudioTransferBufDataWithSyncHdr_DTS;
		new_opcodes[18] = (int)InterfacerContainerAeCopyData_DTS;

		patch_opcodes(hExeDsp, "gAvfdInterfaceAudioCodecFunctions", 2 * 25,
					  old_opcodes, new_opcodes, NULL, 25,
					  PROT_READ | PROT_WRITE);
	}

	/* modify avfd interfacer container functions */
	lib_export = (typeof(lib_export))
		dlsym(hExeDsp, "gAvfdInterfaceContainerFunctions");
	if (lib_export != NULL) {
		memcpy(old_opcodes, lib_export + 7 * 17, sizeof(*lib_export) * 17);
		memcpy(new_opcodes, lib_export + 7 * 17, sizeof(*lib_export) * 17);

		new_opcodes[2] = (int)uldAvfd_MkvDemux_DTS;

		patch_opcodes(hExeDsp, "gAvfdInterfaceContainerFunctions", 7 * 17,
					  old_opcodes, new_opcodes, NULL, 17,
					  PROT_READ | PROT_WRITE );
	}

	dlclose(hExeDsp);

	return (EX_OK);
	
}

const struct mod_config_var *
mod_get_config_vars(void)
{
	return (config_vars);
}

int
mod_keypressed(const int *keys, int keys_num, int *key_out)
{
	if (!config.audio_switch_key || keys[0] != config.audio_switch_key)
		return (MOD_KEYPRESSED_UNKNOWN);

	return (dca_process_keyproc());
}
