/*
 * ffmpeg_mpegts.h
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
#ifndef __FFMPEG_MPEGTS_H__
#define __FFMPEG_MPEGTS_H__

#include <libavformat/mpegts.h>

/* The code below was taken from mpegts.c */

/* TS stream handling */

enum MpegTSState {
    MPEGTS_HEADER = 0,
    MPEGTS_PESHEADER,
    MPEGTS_PESHEADER_FILL,
    MPEGTS_PAYLOAD,
    MPEGTS_SKIP,
};

/* enough for PES header + length */
#define PES_START_SIZE  6
#define PES_HEADER_SIZE 9
#define MAX_PES_HEADER_SIZE (9 + 255)


typedef struct PESContext {
	int pid;
	int pcr_pid; 
	int stream_type;
	MpegTSContext *ts;
	AVFormatContext *stream;
	AVStream *st;
	AVStream *sub_st; 
	enum MpegTSState state;
	/* used to get the format */
	int data_index;
	int total_size;
	int pes_header_size;
	int extended_stream_id;
	int64_t pts, dts;
	int64_t ts_packet_pos; 
	uint8_t header[MAX_PES_HEADER_SIZE];
	uint8_t *buffer;
} PESContext;

#endif
