#include <stdio.h>
#include <sysexits.h>
#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>

#define LOG(fmt, ...)											\
	printf("%s:%d:" fmt "\n", __PRETTY_FUNCTION__, __LINE__,	\
		   ##__VA_ARGS__)

static void
my_av_log_callback(void* ptr, int level, const char* fmt, va_list vl)
{
	vprintf(fmt, vl);
}

int
main(int argc, char **argv)
{
    struct AVFormatContext *av_format_context = NULL;
	int rc;

	if (argc <= 1) {
		LOG("Pass path to the media file");
		return (EX_USAGE);
	}
/* 	av_log_set_callback(my_av_log_callback); */
	av_register_all();
	if ((rc = av_open_input_file(&av_format_context,
								 argv[1], NULL, 0, NULL)) != 0) {
		LOG("Error opening file[%s]: %d", argv[1], rc);
		return (EX_DATAERR);
	}
	LOG("File[%s] opened[%p]", argv[1], av_format_context);
	av_format_context->debug = -1;

	if ((rc = av_find_stream_info(av_format_context)) < 0) {
		LOG("Error finding stream info: %d", rc);
		av_close_input_file(av_format_context);
		return (EX_UNAVAILABLE);
	}
	LOG("Stream info found, dumping");
	dump_format(av_format_context, 0, argv[1], 0);
	LOG("Bye");
	av_close_input_file(av_format_context);

	return (EX_OK);
}
