/*
 * uldAvfdInterfacer.h
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */ 

#ifndef __ULDAVFDINTERFACER_H__
#define __ULDAVFDINTERFACER_H__

#include <stdio.h>
#include <stdint.h>
	
#include "Config.h"
#include "OsaSem.h"
#include "OsaTask.h"


#define MAX_AVFD_AUDIO_CODECS 9
#define MAX_AVFD_INSTANCES 1

#define AVFD_INTERFACE_MAGIC 0x20050823


/* forward declarations */
struct avfd_instance_t;
struct avfd_interface_data_t;

/* audio driver */
struct AVFD_ContainerInterface_t {
	int  (*ParseHeader)(void *);					// 00
	int  (*Func01)(void *);							// 01
	int  (*Demux)(void *);							// 02
	int  (*DemuxStop)(void *);						// 03
	int  (*DemuxPause)(void *);						// 04
	int  (*GetAvInfo)(void *);						// 05
	int  (*ChangeAudioStreamId)(struct avfd_instance_t *, int, int);	// 06
	int  (*ChangeVideoStreamId)(void *);			// 07
	int  (*Func08)(void *);							// 08
	int  (*SetSCR)(void *);							// 09
	int  (*IsTimePlaySupported)(void *);			// 10
	int  (*GetFileOffsetFromTime)(void *);			// 11
	int  (*Func12)(void *);							// 12
	int  (*GetPos)(struct avfd_instance_t *, uint64_t *);				// 13
	int  (*Func15)(void *);							// 14
	int  (*Func16)(void *);							// 15
	int  (*IsStopped)(void *);						// 16
};
typedef struct AVFD_ContainerInterface_t AVFD_ContainerInterface_t;

/* audio driver */
struct AVFD_AudioInterface_t {
	int  (*Open)(int, struct avfd_interface_data_t **);					// 00
	int  (*Close)(void *);												// 01
	int  (*AudioTransferData)(struct avfd_instance_t *, int, int);		// 02
	int  (*AudioTransferDataWithPESHdr)(void *);	// 03
	int  (*AudioTransferBufDataWithPESHdr)(void *);	// 04
	int  (*Func05)(void *);							// 05
	int  (*IsAudStreamEnd)(void *);					// 06
	int  (*RegisterCallback)(void *);				// 07
	int  (*UnregisterCallback)(void *);				// 08
	int  (*Func09)(void *);							// 09
	int  (*GetBufferPtr)(void *);					// 10
	int  (*Func11)(void *);							// 11
	int  (*Func12)(void *);							// 12
	int  (*SetWritePtr)(struct avfd_interface_data_t *, int);		// 13
	int  (*AudioBuffInitEx)(void *);				// 14
	int  (*Func15)(void *);							// 15
	int  (*Func16)(void *);							// 16
	int  (*Func17)(void *);							// 17
	int  (*InterfacerContainerAeCopyData)(struct avfd_instance_t *, unsigned char *, unsigned int);	// 18
	int  (*SetStartingStc)(void *);					// 19
	int  (*Func20)(void *);							// 20
	int  (*Func21)(void *);							// 21
	int  (*Func22)(void *);							// 22
	int  (*Func23)(void *);							// 23
	int  (*Func24)(void *);							// 24
};
typedef struct AVFD_AudioInterface_t AVFD_AudioInterface_t;

struct AVFD_VideoInterface_t {
	int  (*Open)(struct avfd_instance_t *);			// 00
	int  (*Close)(void *);							// 01
	int  (*VideoTransferData)(void *);				// 02
	int  (*VideoTransferDataWithPESHdr)(void *);	// 03
	int  (*VideoTransferBufDataWithPESHdr)(void *);	// 04
	int  (*ParseParam)(void *);							// 05
	int  (*IsVidStreamEnd)(void *);					// 06
	int  (*Func07)(void *);				// 07
	int  (*Func08)(void *);				// 08
	int  (*Func09)(void *);								// 09
	int  (*Func10)(void *);								// 10
	int  (*GetReadPtr)(void *);							// 11
	int  (*GetWritePtr)(void *);						// 12
	int  (*SetWritePtr)(int, int);					// 13
	int  (*BuffInit)(void *);						// 14
	int  (*BuffReset)(void *);						// 15
	int  (*SetDecodingMode)(void *);				// 16
	int  (*SetSpeedLevel)(void *);					// 17
	int  (*InterfacerContainerCopyData)(struct avfd_instance_t *, unsigned char *, unsigned int);	// 18
	int  (*SetStcValue)(void *);					// 19
	int  (*GetPtsValue)(void *);					// 20
	int  (*SyncOnOff)(void *);						// 21
	int  (*Func22)(void *);							// 22
	int  (*Func23)(void *);							// 23
	int  (*BuffLevel)(void *);						// 24
};
typedef struct AVFD_VideoInterface_t AVFD_VideoInterface_t;

struct AVFD_SubtitleInterface_t {
	int  (*Open)(struct avfd_instance_t *, unsigned char **);	// 00
	int  (*Close)(void *);							// 01
	int  (*TransferData)(void *);				// 02
	int  (*TransferDataWithPESHdr)(void *);	// 03
	int  (*TransferBufDataWithPESHdr)(void *);	// 04
	int  (*Func05)(void *);						// 05
	int  (*Func06)(void *);						// 06
	int  (*Func07)(void *);						// 07
	int  (*Func08)(void *);						// 08
	int  (*SubtGetWritePtr)(void *);			// 09
	int  (*Func10)(void *);						// 10
	int  (*GetReadPtr)(void *);					// 11
	int  (*GetWritePtr)(void *);				// 12
	int  (*SetWritePtr)(int, int);				// 13
	int  (*BuffInit)(void *);					// 14
	int  (*BuffReset)(void *);					// 15
	int  (*Func16)(void *);						// 16
	int  (*Func17)(void *);						// 17
	int  (*Func18)(void *);						// 18
	int  (*Func19)(void *);						// 19
	int  (*Func20)(void *);						// 20
	int  (*Func21)(void *);						// 21
	int  (*Func22)(void *);						// 22
	int  (*Func23)(void *);						// 23
	int  (*Func24)(void *);						// 24
};
typedef struct AVFD_SubtitleInterface_t AVFD_SubtitleInterface_t;

struct avfd_init_t {
	int	mRes0000;
	int	mRes0004;
	int	mRes0008;
	int	mRes000C;
	int	mRes0010;
	int	mRes0014;
	int	mRes0018;
	int	mRes001C;
};

struct avfd_interface_data_t {
	int Magic;
	int mRes004;
	int mRes008;
	int mRes00C;
	int mRes010[4];
	int mRes020[4];
	int mRes030[4];
	int mRes040[4];
	int mRes050[4];
	int mRes060[4];
	int mRes070;
	int mRes074;
	int mRes078;
	int mRes07C;
};


struct avfd_interface_t {
	struct avfd_instance_t * avfd;
	struct avfd_interface_data_t * AvfdInterfaceData;
	int mRes008;
	int Magic;
	int RefCount;
	int mRes014;
	AVFD_VideoInterface_t * VideoCodecFunctions;			// 0x018
	AVFD_AudioInterface_t * AudioCodecFunctions;			// 0x01C
	AVFD_SubtitleInterface_t * SubtitleCodecFunctions;		// 0x020
	int mRes024;
	unsigned char * SubtitleBuffer;
	AVFD_ContainerInterface_t * ContainerFunctions;			// 0x02C
};

struct mkv_video_codec_t {
	int mRes00[4];
	int mRes10[4];
	int mRes20[3];
	void * PrivateData;			// 0x02C
	uint64_t ullPrivateSize;	// 0x030
};

struct mkv_demux_context_t {
	int mRes000[4];
	//struct mkv_video_codec_t * video_codec;
	int mRes010;
	int mRes014;
	int mRes018;
	int mRes01C;
	int mRes020[4];
	uint64_t ts;								// 0x030
	int mRes038;
	int mRes03C;
	int mRes040[4];
	int mRes050[4];
	int mRes060[4];
	int mRes070[4];
	int mRes080[4];
	int mRes090[4];
	int mRes0A0[4];
	int mRes0B0[4];
	int mRes0C0[4];
	int mRes0D0[4];
	int mRes0E0[4];
	int mRes0F0[4];
	int mRes100;
	int mRes104;
	int mRes108;
	struct mkv_video_codec_t * mkv_video_codec;	// 0x010C
};

/* .text:009D8150: sizeof(mkv_control_t) = 0x220 */
struct mkv_control_t {
	FILE * stream;
	int	mRes004;
	int	mRes008;
	int	mRes00C;
	struct mkv_demux_context_t * mkv_demux_ctx;	// 0x010
	unsigned char num_tracks;					// 0x014
	int mRes018;
	int mRes01C;
	int	mRes020[4];
	int64_t embl_size;					// 0x030
	int64_t	mRes038;
	int	mRes040[4];
	int	mRes050[4];
	int	mRes060;
	int	mRes064;
	int	mRes068;
	int	mRes06C;
	int	mRes070;
	int	mRes074;
	int64_t	mRes078;
	int64_t	mRes080;
	int	mRes088;
	int	mRes08C;
	int	mRes090[4];
	int64_t	buffer_position;			// 0x0A0
	int64_t	mRes0A8;
	int64_t	mRes0B0;
	int64_t mRes0B8;					// 0x0B8
	int64_t	mRes0C0;
	int64_t	position;
	int64_t	mRes0D0;
	int64_t	mRes0D8;
	int64_t	mRes0E0;
	int64_t file_position;
	int	mRes0F0;
	int	mRes0F4;
	int	mRes0F8;
	int	mRes0FC;
	int	mRes100;
	int playstate;								// 0x0104
//	int64_t mRes108;							// 0x0108
	int mRes108;
	int mRes10C;
	int mRes110;
	int mRes114;
	unsigned char vol_header[255];				// 0x0118
	unsigned char vol_header_len;				// 0x0217
	int64_t mRes218;
};

/* size of structure is 0x384D8 bytes */
struct avfd_instance_t {
	int	Magic;								// 0x00000
	int ReferenceCount;						// 0x00004
	int FlagUsed;							// 0x00008
	int mRes000C;
	int mRes0010;
	int mRes0014;
	unsigned char * ClipBufAddr;			// 0x00018
	sem_t * SemAvfdMut;						// 0x0001C
	sem_t * SemAvfdStart;					// 0x00020
	staAvfd_DemuxTask_t * TaskAvfdDemux;	// 0x00024
	int mRes0028;
	FILE * StreamHandle;					// 0x0002C
	int mRes0030;
	int mRes0034[0x0D81];
	int mRes03638;
	int mRes0363C;
	int mRes03640[0x348];
	int mRes04350;
	int audio_id;							// 0x04354
	int video_id;							// 0x04358
	int mRes0435C;
	int mRes04360[0x0216];
	unsigned char * AudioBufferBase;				// 0x04BC8
	unsigned char * AudioBufferEnd;					// 0x04BCC
	unsigned char * AudioWritePtr;					// 0x04BD0
	int mRes04BD4[0x4373];
	int mRes159A0;
	int mRes159A4;
	int mRes159A8;
	int mRes159AC;
	void * OsaHandle;						// 0x159B0
	int mRes159B4;
	int mRes159B8;
	int subtitle_codec_id;					// 0x159BC
	int mRes159C0;
	int container_codec_id;					// 0x159C4
	int video_stream_id;					// 0x159C8
	int video_codec_id;						// 0x159CC
	int audio_stream_id;					// 0x159D0
	int audio_codec_id;						// 0x159D4
	int mRes159D8[0x1B];
	int AudioBufOffset;							// 0x15A44
	struct avfd_interface_t * avfd_interface;	// 0x15A48	 
	int mRes15A4C[0x8A21];
	int mRes382D0;
	int mRes382D4;
	int mRes382D8;
	int mRes382DC;
	unsigned int mRes382E0;					// 0x0382E0
	int mRes382E4;
	int64_t mRes382E8;
//	int stc;
	int mRes382F0[4];
	int mRes38300;
	int mRes38304;
	int mRes38308;
	unsigned char PesHeader[0x13];			// 0x3830C
	int mRes38320;
	int HeaderParsed;						// 0x38324
	int mRes38328;
	int mRes3832C;
	int mRes38330[3];
	int mRes3833C;							// 0x3833C
	int mRes38340;
	int mRes38344;
	int mRes38348;
	int mRes3834C;
	int mRes38350[4];
	int mRes38360;
	int mRes38364;
	struct mkv_control_t * mkv_control;		// 0x038368
	int DemuxStarted;						// 0x03836C
	int mRes38370[0x54];
	unsigned int mRes384C0;
	int mRes384C4;
	int mRes384C8;
	int mRes384CC;
	int mRes384D0;
	int mRes384D4;
};

struct avfd_video_es_data_t {
	void * data_buffer;
	unsigned int data_size;
	void * private_data;
	unsigned int private_size;
};

#define AVFD_STATUS(x) (x & 0xFFF)

#ifdef __cplusplus
extern "C" {
#endif 

LIB_EXPORT extern struct avfd_instance_t avfd_instances[MAX_AVFD_INSTANCES];
LIB_EXPORT extern struct AVFD_ContainerInterface_t gAvfdInterfaceContainerFunctions[];
LIB_EXPORT extern struct AVFD_AudioInterface_t gAvfdInterfaceAudioCodecFunctions[MAX_AVFD_AUDIO_CODECS];
LIB_EXPORT extern struct AVFD_VideoInterface_t gAvfdInterfaceVideoCodecFunctions[];
LIB_EXPORT extern struct AVFD_SubtitleInterface_t gAvfdInterfaceSubTitleCodecFunctions[];

PUBLIC int64_t avfd_GetBufPos( FILE * stream );
/* end_of_list()... */
PUBLIC int avfd_SeekBuf( FILE * stream, int64_t offset, int flag );
/* avfd_GetSwap8Buf()...*/
PUBLIC int avfd_GetBuf( FILE * stream, unsigned char * buffer, int buffsize );
PUBLIC int uldAvfd_StoreAudioPTS( int, int64_t, int );

LIB_EXPORT struct avfd_instance_t * avfd_GetInst(int);
LIB_EXPORT struct avfd_instance_t * avfd_GetInstFromHndl( void * );
LIB_EXPORT int avfd_ReadStream( struct avfd_instance_t *, unsigned char * buffer, int buffsize );
PUBLIC int uldAvfd_GetStc( void *, uint64_t * );
PUBLIC int uldAvfd_InterfacerVideoGetPts( struct avfd_instance_t *, unsigned int * );
LIB_EXPORT int avfd_IsStreamingMode( void );

PUBLIC int uldAvfd_InterfacerVideoSetEndofStream( struct avfd_instance_t * );
PUBLIC int uldAvfd_InterfacerAudioSetEndofStream( struct avfd_instance_t * );
PUBLIC int uldAvfd_InterfacerCodecTransVideoLeftOutData( struct avfd_instance_t * );
LIB_EXPORT int uldAvfd_InterfacerCodecTransSubTitleData(int,int,int);
LIB_EXPORT int uldAvfd_InterfacerVideoCodecSetSpeed(int,int);
LIB_EXPORT int uldAvfd_InterfacerVideoCodecSetDecodingMode(int,int);
LIB_EXPORT int uldAvfd_InterfacerContainerSetDecInfo(int);
PUBLIC int uldAvfd_InterfacerContainerIsDemuxStopped(struct avfd_instance_t *);
LIB_EXPORT int uldAvfd_InterfacerContainerAutoConfigure(int);
LIB_EXPORT int uldAvfd_InterfacerContainerGetPos(struct avfd_instance_t *, uint64_t *);
LIB_EXPORT int uldAvfd_InterfacerContainerGetFileOffsetByTime(int,int,int);
LIB_EXPORT int uldAvfd_InterfacerContainerIsIndexValid(int);
LIB_EXPORT int uldAvfd_InterfacerContainerChangeSubPicStrm(int,int,int);
LIB_EXPORT int uldAvfd_InterfacerContainerChangeVidStrm(int,int);
LIB_EXPORT int uldAvfd_InterfacerContainerDemuxerClose(int);
LIB_EXPORT int uldAvfd_InterfacerContainerChangeAudStrm(struct avfd_instance_t *,int,int); // DONE
LIB_EXPORT int uldAvfd_InterfacerContainerParseIndex(int);
LIB_EXPORT int uldAvfd_InterfacerContainerParseHeader(struct avfd_instance_t *);
LIB_EXPORT int uldAvfd_InterfacerIsTrickAllowed(int,int,int,int);
LIB_EXPORT int uldAvfd_ShortHeaderPresent(int,int);
LIB_EXPORT int uldAvfd_InterfacerContainerPause(int,int);
LIB_EXPORT int uldAvfd_InterfacerContainerGetHeaderInfo(int,int);
PUBLIC int uldAvfd_InterfacerContainerDemuxStopRelease(struct avfd_instance_t *);
LIB_EXPORT int uldAvfd_InterfacerClose(int);
LIB_EXPORT int uldAvfd_InterfacerContainerDemuxStopWait(int);
LIB_EXPORT int uldAvfd_InterfacerContainerDemuxStop(struct avfd_instance_t *);
LIB_EXPORT int uldAvfd_InterfacerContainerDemuxStart(struct avfd_instance_t *);
LIB_EXPORT int uldAvfd_InterfacerOpen(struct avfd_instance_t * avfd);
LIB_EXPORT int uldAvfd_MkvPause(int,int);
LIB_EXPORT int uldAvfd_MkvIsStopped(int);
LIB_EXPORT int uldAvfd_MkvIsIndexValid(int);
LIB_EXPORT int uldAvfd_MkvStop(int);
/* LIB_EXPORT int uldAvfd_MkvGetPos(struct avfd_instance_t *, int ); */
LIB_EXPORT int uldAvfd_MkvGetFileOffsetFromTime(int,int,int);
/* LIB_EXPORT int uldAvfd_MkvDemux(int); */
LIB_EXPORT int uldAvfd_MkvSetDecInfo(int);
LIB_EXPORT int uldAvfd_MkvChangeAudioStream(int,int,int);
LIB_EXPORT int uldAvfd_MkvAutoConfigure(int);
LIB_EXPORT int uldAvfd_MkvGetAvInfo(int,int);
LIB_EXPORT int uldAvfd_MkvFree(int);
LIB_EXPORT int uldAvfd_MkvParseHeader(int);
LIB_EXPORT int staAvfd_Mp4GetPTS(int,int,int);
LIB_EXPORT int uldAvfd_Mp4FindInterleavedBufferLen(int);
LIB_EXPORT int staAvfd_Mp4NonIntrEOF(int,int);
LIB_EXPORT int staAvfd_Mp4SetState(int,int);
LIB_EXPORT int uldAvfd_Mp4IsStopped(int);
LIB_EXPORT int uldAvfd_Mp4Pause(int,int);
LIB_EXPORT int uldAvfd_Mp4Stop(int);
LIB_EXPORT int staAvfd_Mp4CancelTrick(int);
LIB_EXPORT int staAvfd_Mp4CancelChangeAudio(int);
LIB_EXPORT int staAvfd_Mp4SwitchToNormal(int,int);
LIB_EXPORT int uldAvfd_Mp4GetFileOffsetFromTime(int,int,int);
LIB_EXPORT int staAvfd_Mp4TransferData(int,int);

LIB_EXPORT int staAvfd_AeTransferData(struct avfd_instance_t *, unsigned int, unsigned int );
PUBLIC int staAvfd_GetAeBufferFreeSize( struct avfd_instance_t *, unsigned int * );
LIB_EXPORT int staAvfd_IsAudStreamEnd( struct avfd_instance_t * avfd, int param2 );

PUBLIC int uldAvfd_InterfacerCodecTransAudioEsData( struct avfd_instance_t *, int , int , int , int );
PUBLIC int uldAvfd_ResetAudioIndexTable( void );
PUBLIC int uldAvfd_InterfacerAudioTransferData( struct avfd_instance_t *, int, int * );
PUBLIC int uldAvfd_InterfacerAudioTransferBufDataWithSyncHdr( struct avfd_instance_t * avfd, unsigned char * param2, int param3, int param4);
PUBLIC int uldAvfd_InterfacerAudioTransferDataWithSyncHdr( struct avfd_instance_t * avfd, int param2, int param3, int param4, int param5);
PUBLIC int uldAvfd_InterfacerContainerAeCopyData( struct avfd_instance_t *, unsigned char *, unsigned int );
PUBLIC int uldAvfd_InterfacerCodecCopyAudioData(struct avfd_instance_t * avfd, unsigned char * buffer, int buffsize);
LIB_EXPORT int uldAvfd_InterfacerCodecTransAudioData(struct avfd_instance_t * avfd, int buffsize, int param3 );
PUBLIC int uldAvfd_InterfacerCodecTransVideoEsData( struct avfd_instance_t * avfd, int, int, int, struct avfd_video_es_data_t * );

LIB_EXPORT int uldAvfd_InterfacerAudioAc3TransferDataWithPESHdr( struct avfd_instance_t * , unsigned int , int , int );
LIB_EXPORT int uldAvfd_InterfacerAudioAc3TransferBufDataWithPESHdr( struct avfd_instance_t * avfd, unsigned char * buffer, unsigned int buffsize, int param4);
LIB_EXPORT int uldAvfd_InterfacerGeneratePesHeader( struct avfd_instance_t *, unsigned int, int, int, int );
LIB_EXPORT int uldAvfd_InterfacerSetVideoCodec( struct avfd_instance_t *, int );
LIB_EXPORT int uldAvfd_InterfacerSetAudioCodec( struct avfd_instance_t *, int );
LIB_EXPORT int uldAvfd_InterfacerSetSubtitleCodec( struct avfd_instance_t *, int );

PUBLIC int uldAvfd_InterfacerSetStc( struct avfd_instance_t *, int );
LIB_EXPORT int staAvfdIsDemuxStop( struct avfd_instance_t * avfd, int mode );

LIB_EXPORT int modAe_GetBufferFreeSize( struct avfd_interface_data_t * avfd_interface_data, unsigned int * );

#ifdef __cplusplus
}
#endif 

#endif
