/*
 * uldAvfdMkvControl.c
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * This file was taken from Audioswitch package and modified in order
 * to work with samygodca
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <memory.h>
#include <arpa/inet.h>

#include "uldAvfdInterfacer.h"
#include "uldAvfdMkvControl.h"
#include "uldAvfdMkvEbml.h"
#include "spIAvfd.h"
#include "Logging.h"

static const unsigned int avc_dw1m = 0x1000000;
static const unsigned char audiotransfer_sync_header[] = {0x7B,0x7A,0x9D,0x0C,0x6F,0x1E,0x4B,0x3A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

/* 009C65E8 */
void staMkv_SetState( struct avfd_instance_t * avfd, int new_state )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_control->playstate != 2 )
		mkv_control->playstate = new_state;

	mkv_control->mRes038 = 0;
	mkv_control->mRes0E0 = 0;
	mkv_control->mRes0D8 = 0;
	mkv_control->embl_size = 0;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);
}

/* 009C6A48 */
void staMkv_CancelAudioChange( struct avfd_instance_t * avfd )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( avfd_SeekBuf(avfd->StreamHandle, mkv_control->mRes0E0, SEEK_SET) )
		staMkv_SetState( avfd, 3 );


	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);
}

/* 009C6B40 */
void staMkv_UpdatePlayMode( struct avfd_instance_t * avfd )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( (avfd->mRes382E0 != mkv_control->mRes10C)  ||
		(avfd->mRes3832C != mkv_control->mRes108) ){
		mkv_control->mRes108 = avfd->mRes3832C;
		mkv_control->mRes10C = avfd->mRes382E0;
		if( spI_CheckPrintModule(0x80000000) )
			PRINTF("[AVFD ] Play mode:%d -- %d", mkv_control->mRes10C, mkv_control->mRes108 );
	}

	if( avfd->mRes03638 ){
		if( mkv_control->mRes10C != 2 ){
			if( mkv_control->mRes074 == 0 ){
				if( spI_CheckPrintModule(0x80000000) )
					PRINTF("[AVFD ] New Audio change");
				mkv_control->mRes074 = 1;
			} else {
				if( spI_CheckPrintModule(0x80000000) )
					PRINTF("[AVFD ] Audio change Interrupted");
				staMkv_CancelAudioChange( avfd );
			}
		}
		avfd->mRes03638 = 0;
	}

	if( (mkv_control->mRes074) &&
		(mkv_control->mRes10C == 2 ) ){
			if( spI_CheckPrintModule(0x80000000) )
				PRINTF("[AVFD ] FF-Play- Audio change cancelled");
		staMkv_CancelAudioChange( avfd );
		mkv_control->mRes074 = 0;
	}

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);
}

/* 009C6DB4 */
void staMkv_CancelTrick( struct avfd_instance_t * avfd )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);
	
	mkv_control->mRes108 = 0;
	avfd->mRes3832C = 0;

	if( spI_CheckPrintModule(0x80000000) )
		PRINTF("[AVFD ] Cancel Trick - Old Pos :%Ld", mkv_control->position);

	if( avfd_SeekBuf(avfd->StreamHandle, mkv_control->position, SEEK_SET) )
		staMkv_SetState( avfd, 3 );

	mkv_control->mRes038 = 0;
	mkv_control->position = 0;
	mkv_control->mRes0C0 = 0;
	mkv_control->embl_size = 0;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);
}

/* 009C6F04 */
int uldAvfd_MkvGetPos(struct avfd_instance_t * avfd, int64_t * position )
{
	uint64_t stc;
	unsigned int pts;
	unsigned int tmp;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( (avfd->mRes382D8 == 1) ||
		(avfd->mRes382E0 == 2) ){
		uldAvfd_InterfacerVideoGetPts( avfd, &pts );
		tmp = pts;
	} else {
		if( avfd->mRes3833C == 1 ){
			tmp = avfd->mRes38300;
		} else {
			uldAvfd_GetStc( avfd->OsaHandle, &stc );
			tmp = (unsigned int)((stc >> 32) | (stc << 1));
		}
	}
	*position = tmp / 90;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return 0;
}

/* 009C79BC */
int staMkv_SearchCluster( struct mkv_control_t * mkv_control )
{
	int64_t	ebml_size;
	unsigned int get_byte = 0;
	unsigned int cluster_id;
	unsigned int SearchSize;
	int retval;
	unsigned char cluster_offset;
	FILE * stream = mkv_control->stream;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( avfd_GetBuf(stream, (unsigned char *)&get_byte, 3) ){
		if( spI_CheckPrintLevel(0x100000) ){
			PRINTF("(ERR)[%s_%d] EOF", __FUNCTION__, __LINE__ ); 
		}
		return 0x8522D00C;
	}
	SearchSize = 3;
	/* change le32 to be32 */
	cluster_id = htonl(get_byte);

	while( 1 ){
		/* get next byte */
		if( avfd_GetBuf(stream, (unsigned char *)&get_byte, 1) ){
			if( spI_CheckPrintLevel(0x100000) ){
				PRINTF("(ERR)[%s_%d] EOF", __FUNCTION__, __LINE__ ); 
			}
			return 0x8522D00C;
		}
		SearchSize++;

		/* update cluster_id with next byte */
		cluster_id = ((cluster_id << 8) | (get_byte & 0xFF));

		if( mkv_control->playstate == 2 ){
			if( spI_CheckPrintLevel(0x200000) )
				PRINTF("<+%s>", __FUNCTION__);
			return 0x0FFF;
		}

		switch( cluster_id ){
		case MATROSKA_ID_CLUSTER:
			mkv_control->buffer_position = avfd_GetBufPos(stream) - cluster_offset;
			retval = uldMkv_ReadEbmlSize( stream, &ebml_size, &cluster_offset );
			if( retval ){
				if( spI_CheckPrintLevel(0x100000) )
					PRINTF("(ERR)[%s_%d] Read ebml size error", __FUNCTION__, __LINE__ );
				return retval;
			}
			if( spI_CheckPrintModule(0x80000000) ){
				PRINTF("[AVFD ] Found cluster Pos:%Ld ( SearchSize:%d)", avfd_GetBufPos(stream), SearchSize);
			}
			mkv_control->embl_size = ebml_size;
			return 0;
		case MATROSKA_ID_SEEKHEAD:
		case MATROSKA_ID_TRACKS:
		case MATROSKA_ID_INFO:
		case MATROSKA_ID_CUES:
		case MATROSKA_ID_TAGS:
		case MATROSKA_ID_ATTACHMENTS:
		case MATROSKA_ID_CHAPTERS:
			retval = uldMkv_ReadEbmlSize( stream, &ebml_size, &cluster_offset );
			if( retval ){
				if( spI_CheckPrintLevel(0x100000) )
					PRINTF("(ERR)[%s_%d] Read ebml size error", __FUNCTION__, __LINE__ );
				return retval;
			}
			if( avfd_SeekBuf(stream, ebml_size, 1) )
				return 0x8522D00C;
			if( spI_CheckPrintModule(0x80000000) ){
				PRINTF("[AVFD ] Skip Id:%x size:%Ld", cluster_id, ebml_size);
			}
			break;
		}

		if( SearchSize & 0xFFFFF )
			OsaWait(2);	
	}
}

/* */
int staMkv_TransferAvcData( struct avfd_instance_t * avfd, int pts, int64_t size )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	int retval;
	unsigned char buf_avc_dw1m[4];
	unsigned char buf_var_60[4];
	struct mkv_video_codec_t * mkv_video_codec = avfd->mkv_control->mkv_demux_ctx->mkv_video_codec;
	struct avfd_video_es_data_t video_es_data;
	void *var_70 = NULL;

	memcpy( buf_avc_dw1m, (void *)&avc_dw1m, 4 );

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	video_es_data.data_buffer = NULL;
	video_es_data.data_size = 0;
	video_es_data.private_data = NULL;
	video_es_data.private_size = 0;

	if( mkv_control->mRes0F0 ){
		//if( (avfd->mkv_control->mkv_video_codec->mRes34 > 0) || 
		//	((avfd->mkv_control->mkv_video_codec->mRes34 == 0) && (avfd->mkv_control->mkv_video_codec->mRes30 < 6)) ){
		if( mkv_video_codec->ullPrivateSize > 6 ){
			memcpy( buf_var_60, (void *)&mkv_video_codec->PrivateData, 4 );
			if( buf_var_60[0] == 1 ){
			} else {
				mkv_control->mRes0F8 = 5;
				var_70 = malloc( mkv_video_codec->ullPrivateSize );
				memcpy( var_70, mkv_video_codec->PrivateData, mkv_video_codec->ullPrivateSize );
			}
		} else {
			/* loc_9C8F10 */
			if( spI_CheckPrintModule(0x80000000) ){
				PRINTF("[AVFD ] Codec information not present:%Ld", mkv_video_codec->ullPrivateSize );
			}
			mkv_control->mRes0F8 = 5;
		}
	}
	if( mkv_control->mRes0F8 > 4 ){
		retval = uldAvfd_InterfacerCodecTransVideoEsData( avfd, size, pts, 0, &video_es_data );
	} else {
		uint64_t pos;
		uint64_t nal_offset;
		unsigned int nal_length;
		unsigned char buf_var_32[4];
		for( pos = 0; pos < size; ){
			if( avfd_GetBuf(avfd->StreamHandle, buf_var_32, mkv_control->mRes0F8) == 0 ){				
				nal_offset = pos + mkv_control->mRes0F8;
				switch( mkv_control->mRes0F8 ){
				case 4:
					nal_length = buf_var_32[0] << 24 | buf_var_32[1] << 16 || buf_var_32[2] << 8 || buf_var_32[3];
					break;
				case 3:
					nal_length = buf_var_32[0] << 24 | buf_var_32[1] << 8 || buf_var_32[2];
					break;
				case 2:
					nal_length = buf_var_32[0] << 8 | buf_var_32[1];
					break;
				default:
					nal_length = buf_var_32[0];
				}
				pos = nal_length + nal_offset;
				if( pos < size ){
					video_es_data.data_buffer = buf_avc_dw1m;
					video_es_data.data_size = 4;
					if( uldAvfd_InterfacerCodecTransVideoEsData(avfd, nal_length, pts, 0, &video_es_data) == 0 ){
						video_es_data.data_buffer = NULL;
						video_es_data.data_size = 0;
						video_es_data.private_data = NULL;
						video_es_data.private_size = 0;
					}
				} else {
					if( size < nal_offset ){
						retval = 0;
					} else {
						if( avfd_SeekBuf(avfd->StreamHandle, 1, size-nal_offset) == 0 ){
							if( spI_CheckPrintLevel(0x100000) )
								PRINTF("%s: Incorrect NAL Length:%d --%Ld, %Ld", __FUNCTION__,
									nal_length, nal_offset, size );
							retval = 0;
						} else {
							retval = 0xFFF;
						}
					}
				}
			} else {
				retval = 0xFFF;
			}
		}
		retval = 0;
	}

	if( var_70 )
		free( var_70 );

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return retval;
}
/* loc_9C904C */
int staMkv_TransferMpeg4Data( struct avfd_instance_t * avfd, int pts, int64_t size )
{
	// int retval;
	struct avfd_video_es_data_t video_es_data;
	struct mkv_control_t * mkv_control = avfd->mkv_control;

	video_es_data.data_buffer = NULL;
	video_es_data.data_size = 0;
	video_es_data.private_data = NULL;
	video_es_data.private_size = 0;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_control->mRes0F0 ){
		mkv_control->mRes0F0 = 0;
		video_es_data.private_data = mkv_control->mkv_demux_ctx->mkv_video_codec->PrivateData;
		video_es_data.private_size = mkv_control->mkv_demux_ctx->mkv_video_codec->ullPrivateSize;
		if( video_es_data.private_size == 0 ){
			if( mkv_control->vol_header_len ){
				if( spI_CheckPrintModule(0x80000000) )
					PRINTF("[AVFD ] Inserting VOL Header");
				video_es_data.data_buffer = mkv_control->vol_header;
				video_es_data.data_size = mkv_control->vol_header_len;
			}
		}
		/* send out */
	}

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);


	PRINTF("not finished!");
	return 0xFFF;

}

/* 009C8BBC */
int staMkv_TransferVideoData( struct avfd_instance_t * avfd, int pts, int64_t size )
{
	int retval = 0;
	struct mkv_control_t * mkv_control = avfd->mkv_control;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_control->mRes070 ){
		uldAvfd_InterfacerSetStc( avfd, pts );
		mkv_control->mRes070 = 0;
	}
	switch( staMkv_GetVideoCodecType(mkv_control->mkv_demux_ctx->mkv_video_codec) ){
	case 0x0B:
		retval = staMkv_TransferAvcData( avfd, pts, size );
		break;
	case 0x05:
		break;
	case 0x0C:
		break;
	case 0x0D:
		break;
	default:
		retval = uldAvfd_InterfacerCodecTransVideoEsData( avfd, size, pts, 0, 0 ); 
	}


	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return retval;
}


int staMkv_SkipData(struct avfd_instance_t * avfd, int64_t offset ) 
{
	int retval;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( avfd_SeekBuf(avfd->StreamHandle, offset, SEEK_CUR) )
		retval = 0x8522D00C;
	else
		retval = 0x0;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);
	
	return retval;
}

int staMkv_IsAudioSkip( struct avfd_instance_t * avfd, struct mkv_frame_t * mkv_frame )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( (avfd->audio_stream_id == mkv_frame->stream_id) &&
		!mkv_control->mRes080 &&
		(!mkv_control->mRes078 || (mkv_control->mRes064 == 0 )) &&
		(!mkv_control->mRes068) &&
		(!mkv_control->mRes06C) &&
		( mkv_control->mRes068 != 2) ){
			/* nothing */
	}
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return 0;
}

int staMkv_TransferWmaData( struct avfd_instance_t * avfd, struct mkv_frame_t * mkv_frame, int stc, int64_t embl_size )
{
	unsigned char temp_header[16];
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	int retval;
	unsigned short wma_size;

	memcpy( temp_header, audiotransfer_sync_header, sizeof(audiotransfer_sync_header) );

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_control->mRes0F4 ){
		memset( temp_header+8, 0xFF, 8 );
		wma_size = mkv_frame->mRes030;
		uldAvfd_InterfacerCodecCopyAudioData( avfd, temp_header, 16 );
		uldAvfd_InterfacerCodecCopyAudioData( avfd, (unsigned char *)&wma_size, 2 );
		uldAvfd_InterfacerCodecCopyAudioData( avfd, mkv_frame->mkv_data, wma_size );
		mkv_control->mRes0F4 = 0;
	}

	retval = uldAvfd_InterfacerCodecTransAudioEsData( avfd, embl_size, stc, 0, 0 );

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return retval;
}

/* 009C9F1C (inlined in staMkv_DumpFrame) */
int staMkv_TransferAudioData( struct avfd_instance_t * avfd, struct mkv_frame_t * mkv_frame, int stc, int64_t embl_size )
{
	int retval;
	struct mkv_control_t * mkv_control = avfd->mkv_control;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_control->mRes070 ){
		uldAvfd_InterfacerSetStc( avfd, stc );
		mkv_control->mRes070 = 0;
	}

	switch( staMkv_GetAudioCodecType(mkv_frame) ){
	case 2:	/* PCM */
		if( ((mkv_frame->mRes03C * mkv_frame->mRes040) >> 3) == embl_size ){
			PRINTF("[AVFD ] PCM Error");
			if( spI_CheckPrintLevel(0x200000) )
				PRINTF("<-%s>", __FUNCTION__);
			return 0x8522D022;
		}
		/* fall through -> default */
	default:
		retval = uldAvfd_InterfacerCodecTransAudioEsData( avfd, embl_size, stc, 0, 0 );
		break;
	case 5:	/* WMA */
		retval = staMkv_TransferWmaData( avfd, mkv_frame, stc, embl_size );
		break;
	}
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);
	return retval;
}

/* 009C9D40 (in staMkv_DumpFrame) */
int staMkv_IsVideoSkip( struct avfd_instance_t * avfd, struct mkv_frame_t * mkv_frame )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( (avfd->video_stream_id == mkv_frame->stream_id) &&
		!mkv_control->mRes078 &&
		(!mkv_control->mRes068) &&
		( mkv_control->mRes108 == 2 ) &&	
		(!mkv_control->mRes074) &&
		(!mkv_control->mRes0D0) ){
		
		mkv_control->mRes0D0 = 0;
		if( mkv_control->file_position ){
			if( avfd_GetBufPos(avfd->StreamHandle) < mkv_control->file_position )
				mkv_control->file_position = 0;
		}
	}
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return 0;
}

/* 009C9BB0 */
int staMkv_DumpFrame( struct avfd_instance_t * avfd, struct mkv_frame_t * mkv_frame, uint64_t param3, uint64_t embl_size )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	double D8;
	float S14;
	int S16;
	int retval;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_frame->mRes020 > 0 )
		mkv_frame->mRes020 = 1;

	D8  = (double)param3 * mkv_frame->mRes020;
	S14 = (float)mkv_control->mkv_demux_ctx->ts / (float)1000000;
	S16  = D8*S14*90.0;

	LOG("%s: result S16 = %i", __FUNCTION__, S16 );

	switch( mkv_frame->mRes004 ){
	case 1:
		staMkv_IsVideoSkip( avfd, mkv_frame );
		mkv_control->mRes0A8 = mkv_control->buffer_position;
		mkv_control->mRes0B8 = param3;
		retval = staMkv_TransferVideoData( avfd, S16, embl_size );
		mkv_control->mRes0B0 = avfd_GetBufPos( avfd->StreamHandle );
		break;
	case 2:
		staMkv_IsAudioSkip( avfd, mkv_frame );
		retval = staMkv_TransferAudioData( avfd, mkv_frame, S16, embl_size );
		uldAvfd_StoreAudioPTS( S16, mkv_control->buffer_position, S16 );
		break;
	default:
		staMkv_SkipData( avfd, embl_size );
		retval = 0;
		break;
	}

//	if( mkv_frame->mRes004 == 2 ){
//		uldAvfd_StoreAudioPTS( S16, mkv_control->buffer_position, S16 );
//	}

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return retval;
}

void uldAvfd_MkvDemux( struct avfd_instance_t * avfd )
{
	struct mkv_control_t * mkv_control = avfd->mkv_control;
	struct avfd_invoke_callback_t invoke_callback;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	if( mkv_control == NULL ){
		if( spI_CheckPrintModule(0x80000000) )
			PRINTF("%s: [AVFD ] Header parsing not done", __FUNCTION__);

		if( spI_CheckPrintLevel(0x200000) )
			PRINTF("<-%s>", __FUNCTION__);

		return;
	}

	uldAvfd_ResetAudioIndexTable();

	mkv_control->mRes078 = 0;
	mkv_control->mRes218 = 0;
	mkv_control->mRes080 = 0;
	mkv_control->mRes0A8 = 0;
	mkv_control->mRes0B0 = 0;
	mkv_control->mRes0B8 = 0;
	mkv_control->mRes0C0 = 0;
	mkv_control->position = 0;
	mkv_control->mRes0D0 = 0;
	mkv_control->mRes0D8 = 0;
	mkv_control->mRes0E0 = 0;
	mkv_control->file_position = 0;
	mkv_control->mRes0F0 = 1;
	mkv_control->mRes0FC = 0;
	mkv_control->mRes070 = 0;
	mkv_control->mRes064 = 0;
	mkv_control->mRes068 = 0;
	mkv_control->mRes06C = 0;
	mkv_control->mRes074 = 0;
	mkv_control->mRes110 = 0;
	mkv_control->mRes114 = 0;
	mkv_control->mRes060 = 0;
	mkv_control->playstate = 0;
	avfd->mRes3834C = 0;
	mkv_control->mRes100 = 0;

	if( avfd->mRes382E8 == (unsigned int)-1 )
		uldAvfd_InterfacerSetStc( avfd, 0 );

	if( spI_CheckPrintModule(0x80000000) )
		PRINTF("[AVFD ] [MKV]AVFD BUF POS IS %Ld> state:%d", avfd_GetBufPos(avfd->StreamHandle), mkv_control->playstate );

	while( 1 ){
		staMkv_UpdatePlayMode( avfd );

		switch( mkv_control->playstate ){
		case 0:
			if( (mkv_control->mRes10C == 2) &&
				(mkv_control->position) ){
				staMkv_CancelTrick( avfd );
			}
			if( (mkv_control->mRes108 == 2) &&
				(mkv_control->mRes10C == 0) )
				staMkv_SwitchToNormal( avfd );

			if( avfd->mRes382E8 == (unsigned int)-1 ){
				int retval;
				if( mkv_control->mRes074 )
					staMkv_ChangeAudio( avfd );
				retval = staMkv_ParseDataFrame(avfd);
				if( retval ){
					if( AVFD_STATUS(retval) == 0x22 ){
						avfd->mRes3834C = 0x33;
						if( spI_CheckPrintModule(0x80000000) )
							PRINTF("[AVFD ] Badly Encoded PCM stream" );
					} else {
						if( spI_CheckPrintModule(0x80000000) )
							PRINTF("[AVFD ] Parse Data Frame Failed-- Going to Wait" );
					}
					staMkv_SetState( avfd, 3 );
				}
				if( avfd->mRes3834C >= 0x32 ){
					if( avfd->mRes382D8 == 0 )
						avfd_StreamDiscard( avfd );
					uldAvfd_InterfacerContainerDemuxStopRelease( avfd );
					if( spI_CheckPrintModule(0x80000000) )
						PRINTF("[AVFD ] Mkv Invoking Error Callback" );
					invoke_callback.callback_id = 6;
					invoke_callback.callback_param = 0;
					StaAvfd_InvokeCallback( &invoke_callback );
					if( spI_CheckPrintLevel(0x200000) )
						PRINTF("<-%s>", __FUNCTION__);
					return;
				}
			} else {
				if( staMkv_TimePlay(avfd) )
					staMkv_SetState( avfd, 3 );
				avfd->mRes382E8 = (unsigned int)-1;
			}
			break;
		case 2:
			uldAvfd_InterfacerContainerDemuxStopRelease( avfd );
			if( spI_CheckPrintLevel(0x200000) )
				PRINTF("<-%s>", __FUNCTION__);
			return;			
		case 3:
			if( (uldAvfd_InterfacerVideoSetEndofStream(avfd) ) ||
				(uldAvfd_InterfacerAudioSetEndofStream(avfd) ) ||
				(avfd->mRes3833C) ){
				uldAvfd_InterfacerCodecTransVideoLeftOutData( avfd );
				if( (mkv_control->mRes108 == 2) &&
					(mkv_control->mRes10C == 0) &&
					(mkv_control->position == 0) ){
					staMkv_SetState( avfd, 0 );
					continue;
				}
				if( (mkv_control->mRes074 == 0) ||
					(mkv_control->mRes0E0) ){
					OsaWait( 5 );
					continue;
				} else {
					staMkv_SetState( avfd, 0 );
					continue;
				}
			} else {
				if( spI_CheckPrintModule(0x80000000) )
					PRINTF("[AVFD ] Invoking end of file callback" );
				uldAvfd_InterfacerContainerDemuxStopRelease( avfd );
				invoke_callback.callback_id = 1;
				invoke_callback.callback_param = 0;
				StaAvfd_InvokeCallback( &invoke_callback );
				if( spI_CheckPrintLevel(0x200000) )
					PRINTF("<-%s>", __FUNCTION__);
				return;			
			}
			break;
		default:
			OsaWait( 5 );
			break;
		}
	}
	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

}

/* 009CE480 */
int staMkv_GetAudioCodecType( struct mkv_frame_t * mkv_frame )
{
	int retval;
	unsigned int codec_param = mkv_frame->codec_param;
	char * codec_name = mkv_frame->codec_name;

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<+%s>", __FUNCTION__);

	/* default */
	retval = 9;
	for( ; ; ){
		if( !strcmp(codec_name, "A_AAC") ||
			!strcmp(codec_name, "A_AAC/MPEG2/LC") ||
			!strcmp(codec_name, "A_AAC/MPEG2/LC/SBR") ||
			!strcmp(codec_name, "A_AAC/MPEG4/LC") ||
			!strcmp(codec_name, "A_AAC/MPEG4/LC/SBR") ){
			retval = 6;
			break;
		}
		if( !strcmp(codec_name, "A_AC3") ){
			retval = 4;
			break;
		}
		if( !strcmp(codec_name, "A_DTS") ){
			retval = 9;
			break;
		}
		if( !strcmp(codec_name, "A_MPEG/L2") ||
			!strcmp(codec_name, "A_MPEG/L1") ){
			retval = 0;
			break;
		}
		if( !strcmp(codec_name, "A_MPEG/L3") ){
			retval = 3;
			break;
		}
		if( !strcmp(codec_name, "A_PCM/INT/LIT") ){
			retval = 2;
			break;
		}
		if( !strcmp(codec_name, "A_MS/ACM") ){
			switch( codec_param ){
			case 0x1:
				retval = 2;
				break;
			case 0x2:
				retval = 1;
				break;
			case 0x6:
			case 0x7:
				retval = 8;
				break;
			case 0x11:
				retval = 1;
				break;
			case 0x50:
			case 0x55:
				retval = 3;
				break;
			case 0x161:
			case 0x162:
			case 0x163:
				retval = 5;
				break;
			case 0x2000:
				retval = 4;
				break;
			}
			break;
		}
		break;
	}

	if( spI_CheckPrintLevel(0x200000) )
		PRINTF("<-%s>", __FUNCTION__);

	return retval;
}
