/*
 * uldAvfdMkvControl.h
 * Copyright (C) 2010 Andreas Fiedler <smartsmurf@users.sourceforge.net>
 *
 * This file is part of samygodca, a free DTS Coherent Acoustics
 * stream decoder plugin for SamyGO extension of Samsung DTV.
 *
 * This file was taken from Audioswitch package and modified in order
 * to work with samygodca
 *
 * samygodca is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * samygodca is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef __ULDAVFDMKVCONTROL_H__
#define __ULDAVFDMKVCONTROL_H__

#include <stdint.h>
#include "libavformat/matroska.h"

#include "uldAvfdInterfacer.h"
#include "Config.h"

struct mkv_frame_t {
	int stream_id;
	int mRes004;
	char * codec_name;
	int mRes00C;
	int mRes010[4];
	double mRes020;
	int mRes028;
	unsigned char * mkv_data;
	unsigned short mRes030;
	int mRes034;
	int mRes038;
	int mRes03C;
	int mRes040;
	int codec_param;
};

#ifdef __cplusplus
extern "C" {
#endif

LIB_EXPORT void staMkv_SetState( struct avfd_instance_t * avfd, int new_state );
LIB_EXPORT int staMkv_SearchCluster( struct mkv_control_t * mkv_control );
LIB_EXPORT int staMkv_GetClusterOffsetByTime( struct avfd_instance_t *, int64_t, int, int );
LIB_EXPORT int staMkv_ProcessFrame( struct avfd_instance_t * , struct mkv_frame_t * , int , unsigned long );
LIB_EXPORT void staMkv_CancelAudioChange( struct avfd_instance_t * avfd );
LIB_EXPORT int staMkv_GetAudioCodecType( struct mkv_frame_t * );
PUBLIC int staMkv_GetVideoCodecType( struct mkv_video_codec_t * );
LIB_EXPORT int staMkv_TransferWmaData( struct avfd_instance_t *, struct mkv_frame_t *, int, int64_t );
LIB_EXPORT int staMkv_TransferMpeg4Data( struct avfd_instance_t * avfd, int pts, int64_t size );
LIB_EXPORT int staMkv_TransferAvcData( struct avfd_instance_t * avfd, int pts, int64_t size );
LIB_EXPORT int staMkv_TransferVideoData( struct avfd_instance_t * avfd, int pts, int64_t size );
LIB_EXPORT int staMkv_IsVideoSkip( struct avfd_instance_t *, struct mkv_frame_t * );
LIB_EXPORT int staMkv_TransferAudioData( struct avfd_instance_t *, struct mkv_frame_t *, int, int64_t );
LIB_EXPORT int staMkv_IsAudioSkip( struct avfd_instance_t *, struct mkv_frame_t *);
LIB_EXPORT int staMkv_DumpFrame( struct avfd_instance_t *, struct mkv_frame_t *, uint64_t , uint64_t );
LIB_EXPORT int staMkv_SkipData(struct avfd_instance_t *, int64_t ) ;
LIB_EXPORT void uldAvfd_MkvDemux( struct avfd_instance_t * avfd );
LIB_EXPORT void staMkv_UpdatePlayMode( struct avfd_instance_t * avfd );
LIB_EXPORT void staMkv_CancelTrick( struct avfd_instance_t * );
LIB_EXPORT int uldAvfd_MkvGetPos(struct avfd_instance_t *, int64_t * );
PUBLIC int staMkv_SwitchToNormal( struct avfd_instance_t * );
PUBLIC int staMkv_ChangeAudio( struct avfd_instance_t * );
PUBLIC int staMkv_ParseDataFrame( struct avfd_instance_t * );
PUBLIC int staMkv_TimePlay( struct avfd_instance_t * );


#ifdef __cplusplus
}
#endif

#endif
