#include <dlfcn.h>
#include "mod_int.h"

/*
 * Get pointers to available symbols.
 * syms: name-NULL-terminated array of symbols.
 */
int
exedsp_load_symbols(struct lib_symbol *syms, void **dlhandle)
{
	struct lib_symbol *sym;
	void *handle;
	int rc = 0;

	if (dlhandle != NULL)
		*dlhandle = NULL;

	if ((handle = dlopen(NULL, RTLD_LAZY)) == NULL) {
		LOG("Cannot dlopen() ourselves");
		return (-1);
	}

	for (sym = syms; sym->name != NULL; sym++) {
		if ((*sym->ptr = dlsym(handle, sym->name)) == NULL) {
			LOG("Cannot find symbol[%s]: %s", sym->name, dlerror());
			rc = -1;
			/* Try to find as many symbols as possible, so don't
			   break here. */
		}
	}

	if (dlhandle == NULL)
		dlclose(handle);
	else
		*dlhandle = handle;

	return (rc);
}

/* Function checks if we're running on CIP device */
bool
exedsp_is_cip(void)
{
	static bool cip = false, answered = false;

	if (answered)
		return (cip);
	answered = true;

	if (config.cip == CIP_YES)
		cip = true;
	else if (config.cip == CIP_NO)
		cip = false;
	else {
		/* config.cip == CIP_AUTO */
		const char *names[] = {
			"_ZN5TCCIM7CipTestEv",
			"_ZN10CViewerApp20CBKeySequenceCIPTestEPKvPv",
			"_ZN14CViewerManager14ProcessCIPTestEv",
			"_ZN6TPACim13WriteCipStoreEttPKh",
			"_ZN6TPACim12ReadCipStoreEttPh",
			"_ZN6TPACim19NotifyCIPTestResultEb",
			"_ZN7ITPCCim14t_WriteCipFlagEttPKh",
			"_ZN7ITPCCim13t_ReadCipFlagEttPh",
			"_ZN7ITPCCim23t_WriteCipPermanentDataEttPKh",
			"_ZN7ITPCCim13WriteCipStoreEttPKh",
			NULL
		};
		void *handle;
		int i;

		cip = false;
		if ((handle = dlopen(NULL, RTLD_LAZY)) == NULL) {
			LOG("Cannot dlopen() ourselves");
			return (false);
		}

		for (i = 0; names[i] != NULL; i++) {
			if (dlsym(handle, names[i]) == NULL)
				continue;
			LOG("Found CIP symbol[%s]", names[i]);
			cip = true;
			break;
		}
		dlclose(handle);
		if (!cip)
			LOG("No CIP symbols found");
	}

	return (cip);
}
