#include <sys/mman.h>
#include <dlfcn.h>
#include <libgen.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "mod_int.h"

#define CONFIG_MODULE_ON "MODULE"

#define MOD_PREPARE_NAME "mod_prepare"
#define MOD_GET_CONFIG_VARS_NAME "mod_get_config_vars"
#define MOD_INIT_NAME "mod_init"
#define MOD_KEYPRESSED_NAME "mod_keypressed"
#define MOD_DEINIT_NAME "mod_deinit"

/* exeDSP internal logging functions */
static void (*spI_SetPrintLevel)(unsigned mask);
static void (*spI_SetPrintModule)(unsigned module_mask);

/* KeyCommon::SendKeyPressInput() */
static unsigned *KeyCommon_SendKeyPressInput;

static char app_path[MAX_PATH];

struct mod_config_var_entry {
	struct mod_config_var info;
	struct module_entry *module;
	TAILQ_ENTRY(mod_config_var_entry) module_link;
};

struct module_entry {
	char fullname[MAX_PATH];
	char shortname[CSTR];
	void *dl_handle;
	func_mod_keypressed keypressed_func;
	bool core;
	TAILQ_HEAD(, mod_config_var_entry) config_vars;

	TAILQ_ENTRY(module_entry) link;
};

static TAILQ_HEAD(, module_entry) module_list = TAILQ_HEAD_INITIALIZER(module_list);

#define KFILL(name) #name, KEY_ ##name
#define KNUMFILL(name) #name, KEY_NUM ##name
#define KSRCFILL(name) #name, KEY_SOURCE_ ##name
struct keyinfo keyinfos[] = {
	{KFILL(RECORD)},
	{KFILL(STOP)},
	{KFILL(PLAY)},
	{KFILL(SCRBACK)},
	{KFILL(PAUSE)},
	{KFILL(SCRFWD)},
	{"FAV.CH", KEY_FAV_CH},
	{KFILL(SOURCE)},
	{KFILL(POWER)},
	{"PUP", KEY_P_UP},
	{"PDOWN", KEY_P_DOWN},
	{"PRE-CH", KEY_PRE_CH},
	{KNUMFILL(0)},
	{KNUMFILL(1)},
	{KNUMFILL(2)},
	{KNUMFILL(3)},
	{KNUMFILL(4)},
	{KNUMFILL(5)},
	{KNUMFILL(6)},
	{KNUMFILL(7)},
	{KNUMFILL(8)},
	{KNUMFILL(9)},
	{KFILL(RET)},
	{KFILL(OK)},
	{KSRCFILL(TV)},
	{"CH.LIST", KEY_CH_LIST},
	{KFILL(CONTENT)},
	{"MEDIA.P", KEY_MEDIA_P},
	{"TTX/MIX", KEY_TTX_MIX},
	{KFILL(VUP)},
	{KFILL(VDOWN)},
	{KFILL(MUTE)},
	{KFILL(MENU)},
	{KFILL(UP)},
	{KFILL(RIGHT)},
	{KFILL(DOWN)},
	{KFILL(LEFT)},
	{KFILL(RED)},
	{KFILL(GREEN)},
	{KFILL(YELLOW)},
	{KFILL(BLUE)},
	{KFILL(TOOLS)},
	{KFILL(INTERNET)},
	{KFILL(EXIT)},
	{KFILL(INFO)},
	{KFILL(GUIDE)},
	{KFILL(SUBT)},
	{NULL, 0}
};

static int button_pressed(int key) __attribute__ ((noinline));
static int
button_pressed(int key)
{
	static int keys[MAX_KEYS_HISTORY];
	static int mod_key_pressed = 0;
	static int nkeys = 0;
	struct module_entry *mod;
	int rc = MOD_KEYPRESSED_UNKNOWN;
	int res_key;

	if (config.mod_key) {
		if (!mod_key_pressed) {
			if (key == config.mod_key) {
				/* Modifier key pressed first time */
				mod_key_pressed = 1;
				return (KEY_NOTHING);
			}
		} else {
			if (key != config.mod_key)
				key |= MKEY;
			mod_key_pressed = 0;
		}
	}

	memmove(keys + 1, keys, sizeof(keys) - sizeof(keys[0]));
	keys[0] = key;
	if (nkeys < sizeof(keys) / sizeof(keys[0]))
		nkeys++;
	TAILQ_FOREACH(mod, &module_list, link) {
		if (mod->keypressed_func == NULL)
			continue;
		res_key = 0;
		rc = mod->keypressed_func(keys, nkeys, &res_key);
		if (rc == MOD_KEYPRESSED_PROCESSED)
			break;
	}
	if (rc == MOD_KEYPRESSED_PROCESSED && res_key != 0)
		return ((res_key & MKEY) ? KEY_NOTHING : res_key);
	return ((key & MKEY) ? KEY_NOTHING : key);
}

static void keypressed_injection(int a, int key, int arg_r2) __attribute__ ((noinline));
static void
keypressed_injection(int a, int key, int arg_r2)
{
	// gcc should put "LDMFD SP!, {R4,LR}" on top, if everything goes as planned
	asm volatile(
		"STMFD   SP!, {R0,R2,R3,R12}\n"
	);

	button_pressed(key);
	
	asm volatile(
		"MOV     R1, R0\n" // remapped keycode
		"MOV     R6, R0\n" // remapped keycode
		"LDMFD   SP!, {R0,R2,R3,R12}\n"
		"LDMFD   SP!, {R4,LR}\n"
		"MOV     R7, R2\n"  //This is original command that we overwrite
		"LDR     R3, [R8]\n"  //This is original command that we overwrite
		"LDR     PC, =(_ZN9KeyCommon17SendKeyPressInputEii + 0x18 )\n"	//return after injection
	);
}

static int
keypressed_injector(void)
{
	void *page;

	if (KeyCommon_SendKeyPressInput == NULL) {
		LOG("KeyCommon::SendKeyPressInput not found, my life is useless");
		return (-1);
	}

 	page = (void *)(((unsigned) KeyCommon_SendKeyPressInput) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	KeyCommon_SendKeyPressInput[4] = 0xE51FF004;
	KeyCommon_SendKeyPressInput[5] = (unsigned)keypressed_injection;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	cacheflush(KeyCommon_SendKeyPressInput+4, 8);
	return (0);
}

static struct module_entry *
module_add(const char *fullname, const char *shortname)
{
	struct module_entry *entry;
	void *handle = NULL;

	TAILQ_FOREACH(entry, &module_list, link) {
		if (strcmp(entry->shortname, shortname))
			continue;
		LOG("Module name[%s] already added, won't add module[%s]",
			shortname, fullname);
		return (NULL);
	}

	if ((handle = dlopen(fullname[0] ? fullname : NULL,
						 RTLD_LOCAL|RTLD_NOW|RTLD_DEEPBIND)) == NULL) {
		LOG("Cannot dlopen() module[%s]: %s", fullname, dlerror());
		return (NULL);
	}

	entry = calloc(1, sizeof(*entry));
	strlcpy(entry->fullname, fullname, sizeof(entry->fullname));
	strlcpy(entry->shortname, shortname, sizeof(entry->shortname));
	entry->dl_handle = handle;
	TAILQ_INIT(&entry->config_vars);
	TAILQ_INSERT_TAIL(&module_list, entry, link);

	return (entry);
}

static int
read_config_module_list(char *name, char *value, void *arg)
{
	struct module_entry entry;
	const char *path = (typeof(path))arg;
	char *shortname;
	int len;

	if (strcasecmp(name, CONFIG_MODULE_ON))
		return (EX_OK);

#define MOD_POSTFIX ".so"
#define MOD_POSTFIX_LEN (sizeof(MOD_POSTFIX) - 1)
	if ((shortname = basename(value)) == NULL) {
		LOG("basename() error for[%s]", name);
		return (EX_OK);
	}
	if (shortname != value)
		strlcpy(entry.fullname, value, sizeof(entry.fullname));
	else
		snprintf(entry.fullname, sizeof(entry.fullname),
				 "%s/%s", path, shortname);
	len = strlen(shortname);
	if (len <= MOD_POSTFIX_LEN ||
		strcmp(shortname + len - MOD_POSTFIX_LEN, MOD_POSTFIX)) {

		LOG("Incorrect module file name[%s]", shortname);
		return (EX_OK);
	}
	snprintf(entry.shortname, sizeof(entry.shortname),
			 "%.*s", len - MOD_POSTFIX_LEN, shortname);
	module_add(entry.fullname, entry.shortname);

	return (EX_OK);
}

static struct mod_config_var_entry *
mod_get_variable(const char *name)
{
	struct module_entry *module;
	struct mod_config_var_entry *var;

	TAILQ_FOREACH(module, &module_list, link)
		TAILQ_FOREACH(var, &module->config_vars, module_link)
			if (!strcasecmp(var->info.name, name))
				return (var);
	return (NULL);
}

static int
read_config_generic(char *name, char *value, void *arg)
{
	struct mod_config_var_entry *var;
	func_mod_cfg_fill cb;
	
	if ((var = mod_get_variable(name)) == NULL) {
#if 0
		LOG("Unknown variable[%s]",	name);
#endif
		return (EX_OK);
	}

	cb = var->info.action(CFG_ACTION_FILL);
	return (cb(name, value, var->info.ptr, var->info.ctl));
}

static char *
skip_spaces(char *ptr)
{
	while (isspace(*ptr))
		ptr++;
	return (ptr);
}

static int
read_config(FILE *cfg, int (*process)(char *name, char *value, void *arg), void *arg)
{
	char buf[BCSTR];
	char *name, *val, *ptr;
	int i;

	rewind(cfg);

	while (fgets(buf, sizeof(buf), cfg) != NULL) {
		for (i = 0; buf[i] != '\0'; i++) {
			if (buf[i] != '\r' && buf[i] != '\n')
				continue;
			buf[i] = '\0';
			break;
		}
		name = skip_spaces(buf);

		/* skip comment lines or lines with unknown format */
		if (strchr(";#/=", *name) != NULL) {
			continue;
		}
		if ((val = strchr(name, '=')) == NULL) {
			continue;
		}

		for (ptr = val - 1; ptr >= name && isspace(*ptr); ptr--);
		ptr[1] = '\0';

		val = skip_spaces(val + 1);
		for (ptr = val + strlen(val) - 1; ptr >= val && isspace(*ptr); ptr--);
		ptr[1] = '\0';

		process(name, val, arg);
	}
	return (EX_OK);
}

static int
mod_free(struct module_entry *module)
{
	struct mod_config_var_entry *var;

	LOG("Unloading module[%s]", module->shortname);
	if (module->dl_handle != NULL) {
		func_mod_deinit fn;

		if ((fn = dlsym(module->dl_handle, MOD_DEINIT_NAME)) != NULL)
			fn(module->dl_handle);
		dlclose(module->dl_handle);
	}
	while ((var = TAILQ_FIRST(&module->config_vars)) != NULL) {
		TAILQ_REMOVE(&module->config_vars, var, module_link);
		free(var);
	}
	TAILQ_REMOVE(&module_list, module, link);
	free(module);

	return (EX_OK);
}

static int
mod_prepare_all(void)
{
	struct module_entry *module, *tmodule;
	func_mod_prepare fn;
	int rc;

	TAILQ_FOREACH_SAFE(module, &module_list, link, tmodule) {
		if ((fn = dlsym(module->dl_handle, MOD_PREPARE_NAME)) == NULL)
			continue;
		if ((rc = fn(module->dl_handle)) != EX_OK) {
			LOG("%s() returned[%d], unloading module[%s]",
				MOD_PREPARE_NAME, rc, module->shortname);
			mod_free(module);
		}
	}
	return (EX_OK);
}

static int
mod_init_all(void)
{
	struct module_entry *module, *tmodule;
	func_mod_init fn;
	int rc;

	TAILQ_FOREACH_SAFE(module, &module_list, link, tmodule) {
		module->keypressed_func = dlsym(module->dl_handle,
										MOD_KEYPRESSED_NAME);
		if ((fn = dlsym(module->dl_handle, MOD_INIT_NAME)) == NULL) {
			LOG("Module[%s] initialized with no init function",
				module->shortname);
		} else if ((rc = fn(module->dl_handle)) != EX_OK) {
			LOG("%s() returned[%d], unloading module[%s]",
				MOD_PREPARE_NAME, rc, module->shortname);
			mod_free(module);
			continue;
		} else 
			LOG("Module[%s] initialized", module->shortname);
	}
	return (EX_OK);
}

static int
mod_get_config_vars_all(void)
{
	struct module_entry *module, *tmodule;
	struct mod_config_var_entry *var;
	const struct mod_config_var *infos, *info;
	func_mod_get_config_vars fn;

	TAILQ_FOREACH_SAFE(module, &module_list, link, tmodule) {
		if ((fn = dlsym(module->dl_handle, MOD_GET_CONFIG_VARS_NAME)) == NULL)
			continue;
		if ((infos = fn()) == NULL)
			continue;
		for (info = infos; info->name[0] != '\0'; info++) {
			if ((var = mod_get_variable(info->name)) != NULL) {
				LOG("Variable[%s] already registered by module[%s], "
					"unloading module[%s]",
					info->name, var->module->shortname, module->shortname);
				break;
			}
			if (info->action == NULL ||
				info->action(CFG_ACTION_FILL) == NULL) {
				LOG("Empty action or fill function "
					"for variable[%s] module[%s], "
					"unloading the module",
					info->name, module->shortname);
				break;
			}
			var = calloc(1, sizeof(*var));
			var->info = *info;
			var->module = module;
			TAILQ_INSERT_TAIL(&module->config_vars, var, module_link);
		}
		if (info->name[0] != '\0')
			mod_free(module);
	}

	return (EX_OK);
}

static int
dump_config_vars(void)
{
	struct module_entry *module;
	struct mod_config_var_entry *var;
	char buf[BCSTR];
	int rc;
	func_mod_cfg_print cb;

	LOG("Dumping configuration variables");
	TAILQ_FOREACH(module, &module_list, link) {
		TAILQ_FOREACH(var, &module->config_vars, module_link) {
			if ((cb = var->info.action(CFG_ACTION_PRINT)) == NULL ||
				(rc = cb(buf, sizeof(buf),
						 var->info.ptr, var->info.ctl)) != EX_OK)
				LOG("%s::%s", module->shortname, var->info.name);
			else
				LOG("%s::%s = [%s]", module->shortname, var->info.name, buf);
		}
	}
	return (EX_OK);
}

static int
core_mod_init(void)
{
	struct module_entry *entry;
	char buf[MAX_PATH];
	FILE *cfg;

	/*
     * Initialize the logging. It can be disabled later, in
     * mod_init().
     */
	snprintf(buf, sizeof(buf), "%s%s", app_path, "app.log");
	samyapp_log_init(buf);

	snprintf(buf, sizeof(buf), "%s%s", app_path, CONFIG_FNAME);
	LOG("Loading configuration file[%s]", buf);
	if ((cfg = fopen(buf, "r")) == NULL)
		LOG("Cannot open configuration file[%s]", buf);

	entry = module_add("", CORE_MODULE_NAME);
	entry->core = true;

	/* Load modules list only */
	if (cfg != NULL)
		read_config(cfg, read_config_module_list, (void *)app_path);

	mod_prepare_all();

	/* Get modules configuration parameters */
	mod_get_config_vars_all();

	/* Load configuration parameters from the file */
	if (cfg != NULL)
		read_config(cfg, read_config_generic, NULL);

	if (!config.debug) {
		LOG("Logging disabled");
		samyapp_log_deinit();
	}

	dump_config_vars();

	mod_init_all();

	if (cfg != NULL)
		fclose(cfg);
	return (EX_OK);
}

int
core_init(const char *path)
{
	struct lib_symbol syms[] = {
		{SYMENTRY("spI_SetPrintLevel", &spI_SetPrintLevel)},
		{SYMENTRY("spI_SetPrintModule", &spI_SetPrintModule)},
		{SYMENTRY("_ZN9KeyCommon17SendKeyPressInputEii", &KeyCommon_SendKeyPressInput)},
		{SYMENTRY(NULL, NULL)}
	};

	strlcpy(app_path, path, sizeof(app_path));

	exedsp_load_symbols(syms, NULL);

	if (core_mod_init() < 0 || keypressed_injector() < 0)
		return (-1);

	if ((config.mod_key & MKEY) || config.mod_key < 0) {
		LOG("Invalid modifier key[0x%X], "
			"will use the default value[0x%X]", config.mod_key,
			KEY_GREEN);
		config.mod_key = KEY_GREEN;
	}

	if (config.debug && config.exedsp_debug) {
		samyapp_log_stdout();
		/* With printlevel=0xffffffff the TV will hang: too many logging */

		if (!spI_SetPrintLevel || !spI_SetPrintModule) {
			LOG("spI_SetPrintLevel[%p] and/or spI_SetPrintModule[%p] not present",
				spI_SetPrintLevel, spI_SetPrintModule);
		}
		if (spI_SetPrintLevel)
			spI_SetPrintLevel(config.exedsp_severity);
		if (spI_SetPrintModule)
			spI_SetPrintModule(config.exedsp_facility);
	}

	return (0);
}

void
cacheflush(void *start, size_t size)
{
   // For details, please see the Linux sources, files
   // arch/arm/kernel/traps.c, arch/arm/kernel/entry-common.S and
   // asm-arm/unistd.h

   // This syscall is supported by all 2.4 and 2.6 Linux kernels.  2.2 linux
   // kernels got first support with version 2.2.18.

   asm("mov r0, %0\n"
       "mov r1, %1\n"
       "mov r2, #0\n"
	// EABI or Thumb syscall: syscall number passed in 'r7' (syscall base
	// number is 0x0).  Note that Thumb and EABI are generally not the
	// same.  It just happens that the simple cacheflush syscall doesn't
	// expose any of differences in calling conventions.
       "mov r7, #0xf0000\n"
       "add r7, #0x02\n"
       "swi #0\n" ::
       "g"(start), "g"((long)start+size) :
       "r0", "r1", "r2", "r7");
}

const char *
get_app_path(void)
{
	return (app_path);
}

int
key2str(char *buf, size_t bufsize, int key)
{
	struct keyinfo *info;
	char str_mod[256];
	int modpos = 0;
	struct {
		char ch;
		int flag;
	} mods[] = {
		{'M', MKEY},
		{'T', TKEY},
		{0,      0}
	}, *mod;
	int common_flags = 0;

	for (mod = mods; mod->ch != '\0'; mod++) {
		common_flags |= mod->flag;
		if (!(key & mod->flag))
			continue;
		str_mod[modpos++] = mod->ch;
		str_mod[modpos++] = '+';
	}
	str_mod[modpos] = '\0';

	key &= ~common_flags;
	for (info = keyinfos; info->name != NULL; info++) {
		if (key != info->key)
			continue;
		snprintf(buf, bufsize, "%s%s", str_mod, info->name);
		return (EX_OK);
	}
	snprintf(buf, bufsize, "%s0x%X", str_mod, key);
	return (EX_OK);
}

/*
 * Patch code in the memory.
 * (i) handle
 *     Library handle
 * (i) symname
 *     Symbol name int the library
 * (i) opcode_offset
 *     Offset from the beginning of the symbol pointer, in words
 * (i) old_opcodes
 *     Verify current memory by comparing it with old_opcodes array.
 *     If NULL then no check will be performed.
 * (i) new_opcodes
 *     Opcodes to write
 * (o) save_old_opcodes
 *     If not NULL, write there current opcodes before writing
 * (i) num_opcodes
 *     Size of opcodes arrays
 * (i) access_mode
 *     The third argument to mprotect() to restore the original access
 *     mode (for example, PROT_READ | PROT_EXEC).
 */
int
patch_opcodes(void *handle, const char *symname, int opcode_offset,
			  const unsigned int *old_opcodes,
			  const unsigned int *new_opcodes,
			  unsigned int *save_old_opcodes,
			  int num_opcodes, int access_mode)
{
	unsigned int exe_page;
	unsigned int *pfunc;
	int bufsiz = num_opcodes * sizeof(*new_opcodes);

	pfunc = dlsym(handle, symname);
	if (pfunc == NULL){
		LOG("Cannot find[%s]: %s", symname, dlerror());
		return (EX_SOFTWARE);
	}

	if (old_opcodes && memcmp(pfunc + opcode_offset, old_opcodes, bufsiz)) {
		LOG("Patch for[%s] either already applied or different code, "	\
			"can't patch this", symname);
		return (EX_SOFTWARE);
	}

	if (save_old_opcodes)
		memcpy(save_old_opcodes, pfunc + opcode_offset, bufsiz);

	exe_page = (unsigned int)(pfunc + opcode_offset) & ~0xFFF;
	mprotect((void *)exe_page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	memcpy(pfunc + opcode_offset, new_opcodes, bufsiz);
	mprotect((void *)exe_page, 8192, access_mode); 
	cacheflush(pfunc + opcode_offset, bufsiz); 
	return (EX_OK);
}
