#ifndef COMMON_H
#define COMMON_H

#define EXTERN_C_START extern "C" {
#define EXTERN_C_END }

#ifndef __STDC_CONSTANT_MACROS
#define __STDC_CONSTANT_MACROS
#endif

#ifdef __cplusplus
EXTERN_C_START
#endif

#define HIDDEN __attribute__ ((visibility("hidden")))
#define PUBLIC __attribute__ ((visibility("default")))

#include <string.h>
#include <sysexits.h>
#include <stdint.h>
#include "queue.h"

#define MAX_PATH	256
#define CSTR		256
#define BCSTR		(CSTR * 8)

#define HEXFMT "0x%08X"
#define LOG(fmt, ...)												\
	samyapp_log("%s:%d:" fmt "\n", __FUNCTION__, __LINE__,			\
				##__VA_ARGS__)

int samyapp_log(const char *fmt, ... ) __attribute__ ((noinline))
	__attribute__((__format__(printf, 1, 2)));

int samyapp_log_init(char *fname);
int samyapp_log_deinit(void);
int samyapp_log_stdout(void);

int buf2hexstr(const uint8_t *inbuf, size_t inbufsiz,
			   char *outbuf, int outbufsiz);

int strlcpy(char *dst, const char *src, size_t dst_size);
int strlcat(char *dst, const char *src, size_t dst_size);

struct direntry {
	char name[MAX_PATH];
	unsigned char d_type;
	TAILQ_ENTRY(direntry) link;
};
/* struct dirlist */
TAILQ_HEAD(dirlist, direntry);

#define DIRLIST_SORT    0x00000001
#define DIRLIST_DIRONLY 0x00000002
struct dirlist *dirlist_get(char *dir, int flags);
int dirlist_free(struct dirlist *list);

int dir_writable(char *dir);

#ifdef __cplusplus
EXTERN_C_END
#endif

#endif /* COMMON_H */
