#ifndef MODULES_H
#define MODULES_H

#include "common.h"
#include "exedsp.h"
#include "unicode.h"
#include "timezone.h"

#ifdef __cplusplus
EXTERN_C_START
#endif

#define CONFIG_FNAME "app.ini"

/* Touch panel key flag */
#define TKEY 0x0100
/* Modifier key flag */
#define MKEY 0x1000

#define MAX_KEYS_HISTORY 16

/* Keypressed functions return codes */
#define MOD_KEYPRESSED_UNKNOWN   0
#define MOD_KEYPRESSED_PROCESSED 1

/* ===== Key codes ===== */

#define KEY_RECORD	0x49
#define KEY_STOP	0x46
#define KEY_PLAY	0x47
#define KEY_SCRBACK	0x45
#define KEY_PAUSE	0x4A
#define KEY_SCRFWD	0x48
#define KEY_FAV_CH	0x44
#define KEY_SOURCE	0x01
#define KEY_POWER	0x02
#define KEY_P_UP	0x12
#define KEY_P_DOWN	0x10
#define KEY_PRE_CH	0x13
#define KEY_NUM0	0x11
#define KEY_NUM1	0x04
#define KEY_NUM2	0x05
#define KEY_NUM3	0x06
#define KEY_NUM4	0x08
#define KEY_NUM5	0x09
#define KEY_NUM6	0x0A
#define KEY_NUM7	0x0C
#define KEY_NUM8	0x0D
#define KEY_NUM9	0x0E
#define KEY_RET		0x58
#define KEY_OK		0x68
#define KEY_CH_LIST	0x6B
#define KEY_CONTENT	0x79
#define KEY_MEDIA_P	0x8C
#define KEY_TTX_MIX	0x2C
#define KEY_VUP     0x07
#define KEY_VDOWN   0x0B
#define KEY_MUTE    0x0F
#define KEY_MENU    0x1A
#define KEY_UP      0x60
#define KEY_RIGHT   0x62
#define KEY_DOWN    0x61
#define KEY_LEFT    0x65
#define KEY_RED		0x6C
#define KEY_GREEN	0x14
#define KEY_YELLOW	0x15
#define KEY_BLUE	0x16
#define KEY_TOOLS   0x4B
#define KEY_INTERNET 0x93
#define KEY_EXIT    0x2D
#define KEY_INFO    0x1F
#define KEY_GUIDE   0x4F
#define KEY_SUBT    0x25

/* Input sources */
#define KEY_SOURCE_TV        0x1B
#define KEY_SOURCE_EXT1      0x84
#define KEY_SOURCE_EXT2      0xEB
#define KEY_SOURCE_PC        0x69
#define KEY_SOURCE_AV        0xEC
#define KEY_SOURCE_COMPONENT 0x86
#define KEY_SOURCE_HDMI1     0xE9
#define KEY_SOURCE_HDMI2     0xBE
#define KEY_SOURCE_HDMI3     0xC2 /* HDMI3 / DVI */
#define KEY_SOURCE_HDMI4     0xC5

// key codes: non-existing buttons
#define KEY_ASPECT	0x3E
// key codes: non-existing buttons with unknown functions
#define KEY_DISP_TV	0x7D
#define KEY_DISP_VCR	0x7A
#define KEY_DISP_CABLE	0x7B

// key code: nothing happens
#define KEY_NOTHING	0x1C

struct keyinfo {
	char *name;
	int  key;
};

/* See mod.c for filling */
extern struct keyinfo keyinfos[];

void cacheflush(void *start, size_t size);

int patch_opcodes(void *handle, const char *symname, int opcode_offset,
				  const unsigned int *old_opcodes,
				  const unsigned int *new_opcodes,
				  unsigned int *save_old_opcodes,
				  int num_opcodes, int access_mode);

const char *get_app_path(void);

int key2str(char *buf, size_t bufsize, int key);

/*
 * -------------------------------------------------------------
 * Modules part
 * -------------------------------------------------------------
 */

#define CORE_MODULE_NAME "core"

/*
 * All the module functions that may be called by core.
 * Functions names should not have func_ prefix.
 */
typedef int (*func_mod_prepare)(void *handle);
typedef const struct mod_config_var *(*func_mod_get_config_vars)(void);
typedef int (*func_mod_keypressed)(const int *keys, int keys_num, int *key_out);
typedef int (*func_mod_init)(void *handle);
typedef int (*func_mod_deinit)(void *handle);

/*
 * -------------------------------------------------------------
 * Configuration variables related stuff
 * -------------------------------------------------------------
 */

/* Common action function: function returns pointers to other functions */
#define CFG_ACTION_FILL        0
#define CFG_ACTION_PRINT       1
typedef void *(*func_mod_cfg_action)(uint32_t code);

/* All the actions functions prototypes */
typedef int (*func_mod_cfg_fill)(char *name, char *value,
								 void *result, void *ctl);
typedef int (*func_mod_cfg_print)(char *buf, size_t bufsize,
								  void *value, void *ctl);

struct mod_config_var {
	char name[CSTR];
	uint32_t flags;
	func_mod_cfg_action action;
	void *ptr;
	void *ctl;
};

struct cfg_enum_entry {
	char *name;
	int value;
};

struct cfg_stringlist_ctl {
	char *delim;
	int (*check_func)(char *str);
};

struct stringlist_entry {
	char value[CSTR];
	TAILQ_ENTRY(stringlist_entry) link;
};

/* struct stringlist; */
TAILQ_HEAD(stringlist, stringlist_entry);

#define STR_ENTRY(str) core_action_string, (str), (void *)sizeof(str)
#define FLAG_ENTRY(flag) core_action_flag, &(flag), NULL
#define INT_ENTRY(integer) core_action_int, &(integer), NULL
#define SINT_ENTRY(integer) core_action_sint, &(integer), NULL
#define ENUM_ENTRY(integer, ctl) core_action_enum, &(integer), (ctl)
#define STR_LIST_ENTRY(strlist, ctl) core_action_string_list, &(strlist), (void *)(ctl)
#define KEY_ENTRY(integer) core_action_key, &(integer), NULL
#define PATH_ENTRY(str) core_action_path, &(str), (void *)sizeof(str)
#define LAST_VAR_ENTRY "", 0, NULL, NULL, NULL

/* generic variable actions */
void *core_action_int(uint32_t code);
void *core_action_sint(uint32_t code);
void *core_action_flag(uint32_t code);
void *core_action_string(uint32_t code);
void *core_action_enum(uint32_t code);
void *core_action_string_list(uint32_t code);
void *core_action_key(uint32_t code);
void *core_action_path(uint32_t code);

#ifdef __cplusplus
EXTERN_C_END
#endif

#endif /* MODULES_H */
