#include <sys/mman.h>
#include "mod_audio_switch.h"

struct audio_switch_config config = {
	.key = MKEY | KEY_MUTE,
};

static struct mod_config_var config_vars[] = {
	{"AUDIO_SWITCH",    0, KEY_ENTRY(config.key)},
	{LAST_VAR_ENTRY}
};

static int (*CMovieCore_AudioFormatToTD)(int zero, int u8);
static int (*CFeeder_ConvertSdalToAvfdAudFormat)(int zero, int u8);
static int (*spIAvfd_ChangeAudio_p)(int zero, int audio_streamid, int audio_codecid);

static int asw_currentstream = 0;
static int asw_streams = 0;
static int asw_streamid[16];
static int asw_streamcodec[16];
static int asw_streamptr[16];

static char *asw_helper;
static char asw_filename[MAX_PATH];
static int  asw_coremovie_handle;
static int  asw_mediainfo;

static unsigned u7, u8, u9;

static int asw_process_mediatype(void) __attribute__ ((noinline));
static int asw_process_mediatype(void)
{
	LOG("current_media_info_context[0x%X] streamtype[0x%X] "
		"codectype[0x%X] streamid[0x%X]",
		u7, ((unsigned *)u7)[0x238/4], u8, u9);

	u8 = CMovieCore_AudioFormatToTD(0, u8);
	u8 = CFeeder_ConvertSdalToAvfdAudFormat(0, u8);

	LOG("Codectype after convert[0x%X]", u8);
	if ( u8 == 8 )
	{
		/* Unsupported audio, ignore stream */
		((unsigned *)u7)[0x238/4] = 0;
		((unsigned *)u7)[0x23C/4] = 0;
	}
	else
	{
		asw_streamcodec[asw_streams] = u8;
		asw_streamid[asw_streams] = u9;
		asw_streamptr[asw_streams] = u7;
		asw_streams++;
	}

	return 0;
}

static int asw_process_filemoviecore(void) __attribute__ ((noinline));
static int asw_process_filemoviecore(void)
{
	LOG("movie stopped..");
	asw_streams = 0;

	return 0;
}

static int asw_process_mediainfo(void) __attribute__ ((noinline));
static int asw_process_mediainfo(void)
{
	unsigned i, *ptr;

	LOG("coremovie[0x%X] mediainfo[0x%X] filename[0x%X]",
		u7, u9, u8);

	asw_coremovie_handle = u7;
	asw_mediainfo = u9;
	asw_helper = (char *)u8;

	if (!strcmp(asw_filename, asw_helper))
	{
		LOG("Same movie, preparing for setting previous current stream..");
		for (i = 0; i < asw_currentstream; i++)
		{
			/* reset all streams till current stream.. */
			ptr = (unsigned *)asw_streamptr[i];
			
			ptr[0x238/4] = 0;
			ptr[0x23C/4] = 0;
		}
	} else {
		LOG("New movie, resetting current stream..");
		asw_currentstream = 0;
		strlcpy(asw_filename, asw_helper, sizeof(asw_filename));
	}

	return 0;
}

/* It is actually a label defined in asw_mediatype_injection_nocip() */
void asw_mediatype_injection_nocip_lbl(void);

void asw_mediatype_injection_nocip(void) __attribute__ ((noinline));
void asw_mediatype_injection_nocip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_mediatype_injection_nocip_lbl\n"
			"asw_mediatype_injection_nocip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
			"LDR	 R8, [R7,#0x288]\n"
			"ADD	 R9, R6, #0x1\n"
	);

	asm volatile(
		"MOV	r5,%0\n"
		"STR 	r7,[r5]\n"
		"MOV	r5,%1\n"
		"STR 	r8,[r5]\n"
		"MOV	r5,%2\n"
		"STR 	r9,[r5]\n"
		:
		: "r" (&u7), "r" (&u8), "r" (&u9) );

	asw_process_mediatype();

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"ADD     R3, R4, R5\n"		// old replaced instructions
		"LDR     R2, [R3,#0x18]\n"
		"LDR     PC, =(_ZN13CParsingMedia16GetMediaTypeInfoEPKcPN10Multimedia18_tMediaInfoContextE+0x334)\n" );
}

/* It is actually a label defined in asw_mediatype_injection_nocip() */
void asw_mediatype_injection_cip_lbl(void);

void asw_mediatype_injection_cip(void) __attribute__ ((noinline));
void asw_mediatype_injection_cip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
		".global asw_mediatype_injection_cip_lbl\n"
		"asw_mediatype_injection_cip_lbl:\n"
		"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
		"SUB     SP,  SP, #56\n"
		"LDR	 R7, [R8,#0x288]\n"
		"ADD	 R9, R6, #0x1\n"
	);

	asm volatile(
		"MOV    r5,%0\n"
		"STR    r8,[r5]\n"
		"MOV    r5,%1\n"
		"STR    r7,[r5]\n"
		"MOV    r5,%2\n"
		"STR    r9,[r5]\n"
		:
		: "r" (&u7), "r" (&u8), "r" (&u9) );

	asw_process_mediatype();

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"ADD     R3, R4, R5\n"		// old replaced instructions
		"LDR     R2, [R3,#0x18]\n"
		/* base: 0x0FBB2B8 / target: 0x0FBB788 */
		"LDR     PC, =(_ZN13CParsingMedia16GetMediaTypeInfoEPKcPN10Multimedia18_tMediaInfoContextE+0x4D0)\n" );
}

/* It is actually a label defined in asw_filemoviecore_injection_nocip() */
void asw_filemoviecore_injection_nocip_lbl(void);

void asw_filemoviecore_injection_nocip(void) __attribute__ ((noinline));
void asw_filemoviecore_injection_nocip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_filemoviecore_injection_nocip_lbl\n"
			"asw_filemoviecore_injection_nocip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
	);

	asw_process_filemoviecore();

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"MOV     R4, R0\n"		// old replaced instructions
		"SUB     SP, SP, #0xC\n"
		"LDR     PC, =(_ZN14CFileMovieCore4StopEv+0xC)\n" );
}

/* It is actually a label defined in asw_filemoviecore_injection_cip() */
void asw_filemoviecore_injection_cip_lbl(void);

void asw_filemoviecore_injection_cip(void) __attribute__ ((noinline));
void asw_filemoviecore_injection_cip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_filemoviecore_injection_cip_lbl\n"
			"asw_filemoviecore_injection_cip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
	);

	asw_process_filemoviecore();

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"MOV     R4, R0\n"          // old replaced instructions
		"SUB     SP, SP, #0x8\n"
		"LDR     PC, =(_ZN14CFileMovieCore4StopEv+0x18)\n" );
}

/* It is actually a label defined in asw_movieplay_injection_nocip() */
void asw_movieplay_injection_nocip_lbl(void);

void asw_movieplay_injection_nocip(void) __attribute__ ((noinline));
void asw_movieplay_injection_nocip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_movieplay_injection_nocip_lbl\n"
			"asw_movieplay_injection_nocip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
	);

	asm volatile(
		"MOV	R10,%0\n"
		"STR 	R10,[R5,#0xDB0]\n"
		:
		: "r" (asw_streamid[asw_currentstream]) );

	LOG("Setting current stream to[0x%X]", asw_streamid[asw_currentstream]);

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"LDR     R2, [R5,#8]\n"		// old replaced instructions
		"LDR     R1, [R5,#0xDC0]\n"
		"LDR     PC, =(_ZN10Multimedia11MediaPlayer5Movie10CFileMovie4PlayExPhPN11MediaCommon8DRM_INFOE+0x118)\n" );
}


/* It is actually a label defined in asw_movieplay_injection_cip() */
void asw_movieplay_injection_cip_lbl(void);

void asw_movieplay_injection_cip(void) __attribute__ ((noinline));
void asw_movieplay_injection_cip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_movieplay_injection_cip_lbl\n"
			"asw_movieplay_injection_cip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
	);

	asm volatile(
		"MOV	R10,%0\n"
		"STR 	R10,[R5,#0xDB0]\n"
		:
		: "r" (asw_streamid[asw_currentstream]) );

	LOG("Setting current stream to[0x%X]", asw_streamid[asw_currentstream]);

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"LDR     R2, [R5,#8]\n"		// old replaced instructions
		"LDR     R1, [R5,#0xDC0]\n"
		/* base: 0x0FB942C / target: 0x0FB954C */
		"LDR     PC, =(_ZN10Multimedia11MediaPlayer5Movie10CFileMovie4PlayExPhPN11MediaCommon8DRM_INFOE+0x120)\n" );
}

/* It is actually a label defined in asw_mediainfo_injection_nocip() */
void asw_mediainfo_injection_nocip_lbl(void);

void asw_mediainfo_injection_nocip(void) __attribute__ ((noinline));
void asw_mediainfo_injection_nocip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_mediainfo_injection_nocip_lbl\n"
			"asw_mediainfo_injection_nocip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
			"MOV	 R7, R0\n"
			"MOV	 R8, R1\n"
			"MOV	 R9, R2\n"
	);

	asm volatile(
		"MOV	r5,%0\n"
		"STR 	r7,[r5]\n"
		"MOV	r5,%1\n"
		"STR 	r8,[r5]\n"
		"MOV	r5,%2\n"
		"STR 	r9,[r5]\n"
		:
		: "r" (&u7), "r" (&u8), "r" (&u9) );

	asw_process_mediainfo();

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"MOV     R5, R0\n"		// old replaced instructions
		"MOV     R7, R1\n"
		"LDR     PC, =(_ZN14CFileMovieCore4OpenEPcPN10Multimedia18_tMediaInfoContextE+0x5*4)\n" );
}

/* It is actually a label defined in asw_mediainfo_injection_cip() */
void asw_mediainfo_injection_cip_lbl(void);

void asw_mediainfo_injection_cip(void) __attribute__ ((noinline));
void asw_mediainfo_injection_cip(void)
{
	// it's better to use labels, so we can ignore GCC stack instructions
	asm volatile(
			".global asw_mediainfo_injection_cip_lbl\n"
			"asw_mediainfo_injection_cip_lbl:\n"
			"STMFD   SP!, {R0-R12,LR}\n" //Put all General Purpose registers to Stack
			"SUB     SP,  SP, #56\n"
			"MOV	 R7, R0\n"
			"MOV	 R8, R1\n"
			"MOV	 R9, R2\n"
	);

	asm volatile(
		"MOV	r5,%0\n"
		"STR 	r7,[r5]\n"
		"MOV	r5,%1\n"
		"STR 	r8,[r5]\n"
		"MOV	r5,%2\n"
		"STR 	r9,[r5]\n"
		:
		: "r" (&u7), "r" (&u8), "r" (&u9) );

	asw_process_mediainfo();

	asm volatile(
		"ADD     SP, SP, #56\n"
		"LDMFD   SP!, {R0-R12,LR}\n"// Restore all GP registers from Stack
		"MOV     R5, R0\n"		// old replaced instructions
		"MOV     R7, R1\n"
		"LDR     PC, =(_ZN14CFileMovieCore4OpenEPcPN10Multimedia18_tMediaInfoContextE+0x1C)\n" );
}

static void
asw_init_nocip(unsigned *CParsingMedia_GetMediaTypeInfo,
			   unsigned *CFileMovieCore_stop,
			   unsigned *CMovieCore_Open, unsigned *Movie_Play)
{
	void *page;

	/* To load all stream ids and codecs, ignore unsupported */
	page = (void *)(((unsigned)CParsingMedia_GetMediaTypeInfo) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	CParsingMedia_GetMediaTypeInfo[0xCB]=0xE51FF004;
	CParsingMedia_GetMediaTypeInfo[0xCC]=(unsigned) asw_mediatype_injection_nocip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

    /* To clear stream ids and codecs */
 	page = (void *)(((unsigned) CFileMovieCore_stop) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	CFileMovieCore_stop[0x1]=0xE51FF004;
	CFileMovieCore_stop[0x2]=(unsigned) asw_filemoviecore_injection_nocip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	/* To clear stream ids and codecs */
	page = (void *)(((unsigned) CMovieCore_Open) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	CMovieCore_Open[0x3]=0xE51FF004;
	CMovieCore_Open[0x4]=(unsigned) asw_mediainfo_injection_nocip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	page = (void *)(((unsigned) Movie_Play) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	Movie_Play[0xFC/4+5]=0xE51FF004;
	Movie_Play[0xFC/4+6]=(unsigned) asw_movieplay_injection_nocip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	cacheflush(CParsingMedia_GetMediaTypeInfo+0xCB, 8);
	cacheflush(CFileMovieCore_stop+0x1, 8);
	cacheflush(CMovieCore_Open+0x3, 8);
	cacheflush(Movie_Play+0xFC/4+5, 8);
}

static void
asw_init_cip(unsigned *CParsingMedia_GetMediaTypeInfo,
			 unsigned *CFileMovieCore_stop,
			 unsigned *CMovieCore_Open, unsigned *Movie_Play)
{
	void *page;

	/* To load all stream ids and codecs, ignore unsupported */
	page = (void *)(((unsigned)CParsingMedia_GetMediaTypeInfo) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	CParsingMedia_GetMediaTypeInfo[0x132]=0xE51FF004;
	CParsingMedia_GetMediaTypeInfo[0x133]=(unsigned) asw_mediatype_injection_cip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

    /* To clear stream ids and codecs */
 	page = (void *)(((unsigned) CFileMovieCore_stop) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	CFileMovieCore_stop[0x4]=0xE51FF004;
	CFileMovieCore_stop[0x5]=(unsigned) asw_filemoviecore_injection_cip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	/* To clear stream ids and codecs */
	page = (void *)(((unsigned) CMovieCore_Open) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	CMovieCore_Open[0x5]=0xE51FF004;
	CMovieCore_Open[0x6]=(unsigned) asw_mediainfo_injection_cip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	page = (void *)(((unsigned) Movie_Play) & ~0x0FFF);
	mprotect(page, 8192, PROT_READ | PROT_WRITE | PROT_EXEC);
	Movie_Play[0x46]=0xE51FF004;
	Movie_Play[0x47]=(unsigned) asw_movieplay_injection_cip_lbl;
	mprotect(page, 8192, PROT_READ | PROT_EXEC);

	cacheflush(CParsingMedia_GetMediaTypeInfo+0x132, 8);
	cacheflush(CFileMovieCore_stop+0x4, 8);
	cacheflush(CMovieCore_Open+0x5, 8);
	cacheflush(Movie_Play+0x46, 8);
}

int
mod_init(void *handle)
{
	unsigned *CParsingMedia_GetMediaTypeInfo;
	unsigned *CFileMovieCore_stop;
	unsigned *CMovieCore_Open;
	unsigned *Movie_Play;
	struct lib_symbol syms[] = {
		{SYMENTRY("_ZN13CParsingMedia16GetMediaTypeInfoEPKcPN10Multimedia18_tMediaInfoContextE", &CParsingMedia_GetMediaTypeInfo)},
		{SYMENTRY("_ZN14CFileMovieCore4StopEv", &CFileMovieCore_stop)},
		{SYMENTRY("_ZN14CFileMovieCore4OpenEPcPN10Multimedia18_tMediaInfoContextE", &CMovieCore_Open)},
		{SYMENTRY("_ZN10Multimedia11MediaPlayer5Movie10CFileMovie4PlayExPhPN11MediaCommon8DRM_INFOE", &Movie_Play)},

		{SYMENTRY("_ZN10CMovieCore15AudioFormatToTDEN10Multimedia14AudioCodecTypeE", &CMovieCore_AudioFormatToTD)},
		{SYMENTRY("_ZN10CFeeder_CH29locConvertSdalToAvfdAudFormatE15SdAudioFormat_k", &CFeeder_ConvertSdalToAvfdAudFormat)},
 		{SYMENTRY("spIAvfd_ChangeAudio", &spIAvfd_ChangeAudio_p)},
		{NULL, NULL}
	};

	if (exedsp_load_symbols(syms, NULL))
		return (EX_SOFTWARE);

	if (exedsp_is_cip())
		asw_init_cip(CParsingMedia_GetMediaTypeInfo, CFileMovieCore_stop,
					 CMovieCore_Open, Movie_Play);
	else
		asw_init_nocip(CParsingMedia_GetMediaTypeInfo, CFileMovieCore_stop,
					   CMovieCore_Open, Movie_Play);

	return (EX_OK);
}

int
mod_keypressed(const int *keys, int keys_num, int *key_out)
{
	if (!config.key)
		return (MOD_KEYPRESSED_UNKNOWN);

	if (keys[0] != config.key || !asw_streams)
		return (MOD_KEYPRESSED_UNKNOWN);

	asw_currentstream++;

	if (asw_currentstream >= asw_streams) {
		LOG("Returning to the first stream");
		asw_currentstream = 0;
	}

	if (asw_streamid[asw_currentstream]) {
		LOG("Changing stream to[%d] / audio stream id[%d] / audio codec[%d]",
			asw_currentstream, asw_streamid[asw_currentstream],
			asw_streamcodec[asw_currentstream]);

		spIAvfd_ChangeAudio_p(0, asw_streamid[asw_currentstream],
							  asw_streamcodec[asw_currentstream]);
	}
	return (MOD_KEYPRESSED_PROCESSED);
}

const struct mod_config_var *
mod_get_config_vars(void)
{
	return (config_vars);
}
