/*
 * Graphics functions.
 */
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <mod.h>
#include "mod_pvr.h"
#include "icons.h"

#define SCREEN_W	(1920/2)
#define SCREEN_H	(1080/2)
#define SCREEN_BPP	32
#define SCREEN_FLAGS	0

#define WARNING_DELAY	2000
#define NO_DELAY	1000
#define DELETED_DELAY	2000

static SDL_Surface *screen = NULL;
static SDL_Surface *icon = NULL;
static SDL_Rect dstrect;

#define SDL_DECLARE(name) static typeof(name) *p ## name
#define SDL_ENTRY(name) {SYMENTRY(#name, &p ## name)}

SDL_DECLARE(SDL_Init);
SDL_DECLARE(SDL_SetVideoMode);
SDL_DECLARE(SDL_MapRGB);
SDL_DECLARE(SDL_SetColorKey);
SDL_DECLARE(SDL_UpperBlit);
SDL_DECLARE(SDL_Flip);
SDL_DECLARE(SDL_FreeSurface);
SDL_DECLARE(SDL_Quit);
SDL_DECLARE(SDL_Delay);
SDL_DECLARE(IMG_Load);

static int
have_sdl(void)
{
	static enum {
		SDL_LINK_NONE = 0,
		SDL_LINK_FOUND,
		SDL_LINK_NOTFOUND,
	} sdl_link = SDL_LINK_NONE;

	if (sdl_link == SDL_LINK_NONE) {
		struct lib_symbol syms[] = {
			SDL_ENTRY(SDL_Init),
			SDL_ENTRY(SDL_SetVideoMode),
			SDL_ENTRY(SDL_MapRGB),
			SDL_ENTRY(SDL_SetColorKey),
			SDL_ENTRY(SDL_UpperBlit),
			SDL_ENTRY(SDL_Flip),
			SDL_ENTRY(SDL_FreeSurface),
			SDL_ENTRY(SDL_Quit),
			SDL_ENTRY(SDL_Delay),
			SDL_ENTRY(IMG_Load),
			{NULL, NULL}
		};
		if (exedsp_load_symbols(syms, NULL) < 0)
			sdl_link = SDL_LINK_NOTFOUND;
		else
			sdl_link = SDL_LINK_FOUND;
	}
	return (sdl_link == SDL_LINK_FOUND);
}

static int
init_video_mode(void)
{
	if (!config.show_icons || !have_sdl()) {
		screen = NULL;
		return (0);
	}

	pSDL_Init(SDL_INIT_VIDEO);
	screen = pSDL_SetVideoMode(SCREEN_W, SCREEN_H, SCREEN_BPP, SCREEN_FLAGS);
	return (1);
}

static int
set_icon(const char *pngname)
{
   dstrect.x = config.icon_x;
   dstrect.y = config.icon_y;
   if (screen) {
	   char s[MAX_PATH];

	   snprintf(s, sizeof(s), "%s%s.png", get_app_path(), pngname);
	   icon = pIMG_Load(s);
	   if (icon) {
		   Uint32 colorkey = pSDL_MapRGB(icon->format, 0, 0, 0);
		   pSDL_SetColorKey(icon, SDL_SRCCOLORKEY, colorkey);
		   pSDL_UpperBlit(icon, NULL, screen, &dstrect);
		   pSDL_Flip(screen);
		   return 1;
	   }
   }
   return 0;
}

static void
close_video_mode()
{
	if (!have_sdl())
		return;
	if (icon) {
		pSDL_FreeSurface(icon);
		icon = NULL;
	}
	pSDL_Quit();
	screen = NULL;
}

int show_REC(void)
{
   init_video_mode();
   if (screen)
   {
      if (!set_icon(ICON_REC))
      {
         close_video_mode();
         return KEY_DISP_VCR;
      }
      return KEY_NOTHING;
   }
   else close_video_mode();
   return KEY_DISP_VCR;
}
int hide_REC(int recording_deleted)
{
   if (screen)
   {
      close_video_mode();
      if (recording_deleted)
      {
	 init_video_mode();
	 if (screen)
      	 {
            if (set_icon(ICON_DELETED)) pSDL_Delay(DELETED_DELAY);
            close_video_mode();
         }
      }
      return KEY_NOTHING;
   }
   else
   return KEY_DISP_TV;
}

int show_FORBIDDEN(void)
{
   if (screen)
   {
      close_video_mode();

      init_video_mode();
      if (screen)
      {
         if (set_icon(ICON_NO)) pSDL_Delay(NO_DELAY);
         close_video_mode();
         init_video_mode();
         if (screen)
         {
            set_icon(ICON_REC);
         }
      }
      return KEY_NOTHING;
   }
   else close_video_mode();
   return KEY_DISP_CABLE;
}

int show_WARNING(const char *msg)
{
   init_video_mode();
   if (screen)
   {
      if (set_icon(msg)) pSDL_Delay(WARNING_DELAY);
      close_video_mode();
      return KEY_NOTHING;
   }
   else close_video_mode();
   return KEY_DISP_CABLE;
}

int
icon_displayed(void)
{
	return (screen != NULL);
}
